searchData={"items":[{"type":"task","title":"mix compile.surface","doc":"Generate CSS and JS/TS assets for components.","ref":"Mix.Tasks.Compile.Surface.html"},{"type":"task","title":"Setup - mix compile.surface","doc":"Update `mix.exs`, adding the `:surface` compiler to the list of compilers:\n\n```elixir\ndef project do\n  [\n    ...,\n    compilers: [:phoenix] ++ Mix.compilers() ++ [:surface]\n  ]\nend\n```","ref":"Mix.Tasks.Compile.Surface.html#module-setup"},{"type":"task","title":"Configuration (optional) - mix compile.surface","doc":"The Surface compiler provides some options for custom configuration in your `config/dev.exs`.","ref":"Mix.Tasks.Compile.Surface.html#module-configuration-optional"},{"type":"task","title":"Options - mix compile.surface","doc":"* `generate_assets` - instructs the compiler to generate components' css and js files.\n  Set it to `false` when developing a library of components that doesn't require any CSS\n  style nor JS hooks. Default is `true`.\n\n* `hooks_output_dir` - defines the folder where the compiler generates the JS hooks files.\n  Default is `./assets/js/_hooks/`.\n\n* `css_output_file` - defines the css file where the compiler generates the code.\n  Default is `./assets/css/_components.css`.\n\n* `enable_variants` [experimental] - instructs the compiler to generate tailwind variants based\n  on props/data. Currently, only Tailwind variants are supported. Default is `false`.\n  See more in the \"Enabling CSS variants\" section below.\n\n* `variants_output_file` [experimental] - if `enable_variants` is `true`, defines the config file where\n  the compiler generates the scoped variants. Currently, only Tailwind variants are supported.\n  Default is `./assets/css/_variants.js`.\n\n* `variants_prefix` [experimental] - defines a prefix for all variants generated by the compiler.\n  Default is `@`.","ref":"Mix.Tasks.Compile.Surface.html#module-options"},{"type":"task","title":"Example - mix compile.surface","doc":"config :surface, :compiler,\n      hooks_output_dir: \"assets/js/surface\",\n      css_output_file: \"assets/css/surface.css\",\n      enable_variants: true,\n      variants_prefix: \"s-\"","ref":"Mix.Tasks.Compile.Surface.html#module-example"},{"type":"task","title":"Enabling CSS variants - mix compile.surface","doc":"By setting `enable_variants` to `true`, we instruct the compiler to generate tailwind\nvariants based on props/data. All variants are generated in the `variants_output_file`,\nwhich defaults to `./assets/css/_variants.js`.\n\n> **NOTE**: This feature is still experimental and available for feedback.\n> Therefore, the API might change in the next Surface minor version. It's also\n> currently only available for Tailwind.\n\nTo make the generated variants available in your templates, you need to set up the\nproject's `tailwind.config.js` to add the `variants_output_file` as\na preset. Example:\n\n    module.exports = {\n      presets: [\n        require('./css/_variants.js')\n      ],\n      ...\n    }","ref":"Mix.Tasks.Compile.Surface.html#module-enabling-css-variants"},{"type":"task","title":"Defining CSS variants - mix compile.surface","doc":"In order to define CSS variants for your templates, you can use the `css_variant`\noption, which is available for both, `prop` and `data`.","ref":"Mix.Tasks.Compile.Surface.html#module-defining-css-variants"},{"type":"task","title":"Example - mix compile.surface","doc":"prop loading, :boolean, css_variant: true\n    prop size, :string, values: [\"small\", \"medium\", \"large\"], css_variant: true\n\nDepending on the type of the assign you're defining, a set of default variants will\nbe automatically available in your tamplates and be used directly in any `class`\nattribute. By default, all variants names will start with the `@` prefix. If needed,\nyou can change it by setting the `variants_prefix` option.","ref":"Mix.Tasks.Compile.Surface.html#module-example"},{"type":"task","title":"Example - mix compile.surface","doc":"Submit","ref":"Mix.Tasks.Compile.Surface.html#module-example"},{"type":"task","title":"Customizing variants' names - mix compile.surface","doc":"As mentioned in the previous section, each variant name generated bu the compiler\nwill start with a prefix (default is `@`) followed by its base name. Most of the time,\nyou probably want to stick with the default base name, however there are cases when\nrenaming it may be more intuitive. For instance:\n\n    # Name it as `@inactive` instead of `@not-active`\n    prop active, :boolean, css_variant: [false: \"inactive\"]\n\n    # Name it `@valid` and `@invalid` instead of `@has-errors` and `@no-errors`\n    data errors, :list, css_variant: [has_items: \"invalid\", no_items: \"valid\"]\n\n    # Name it `@small`, `@medium` and `@large` instead of `@size-small`, `@size-medium` and `@size-large`\n    prop size, :string, values: [\"small\", \"medium\", \"large\"], css_variant: [prefix: \"\"]\n\nAs you can see, the value of `css_variant` option can be either a boolean or a keyword\nlist of options.\n\nBy passing `true`, the compiler generates variants according to the default values\nfor each option based to the name and type of the related assign. All available options for each type\nare listed below.","ref":"Mix.Tasks.Compile.Surface.html#module-customizing-variants-names"},{"type":"task","title":"Options for `:boolean` - mix compile.surface","doc":"* `:true` - the base name of the variant when the value is truthy. Default is the assign name.\n* `:false` - the base name of the variant when the value is falsy. Default is `not-[assign-name]`.","ref":"Mix.Tasks.Compile.Surface.html#module-options-for-boolean"},{"type":"task","title":"Options for enumerables, e.g. `:list`, `:map` and `:mapset` - mix compile.surface","doc":"* `:has_items` - the base name of the variant when the value list has items.\n  Default is `has-[assign-name]`\n* `:no_items` - the base name of the variant when the value is empty or `nil`.\n  Default is `no-[assign-name]`","ref":"Mix.Tasks.Compile.Surface.html#module-options-for-enumerables-e-g-list-map-and-mapset"},{"type":"task","title":"Options for `:string`, `:atom` and `:integer` defining `values` or `values!` - mix compile.surface","doc":"* `:prefix` - the prefix of the variant's base name generated for each value listed in `values` or `values!`.\n  Default is `[assign-name]-`.","ref":"Mix.Tasks.Compile.Surface.html#module-options-for-string-atom-and-integer-defining-values-or-values"},{"type":"task","title":"Options for other types - mix compile.surface","doc":"* `:not_nil` - the base name of the variant when the value is not `nil`.\n  Default is the assign name.\n* `:nil` - the base name of the variant when the value is `nil`.\n  Default is `no-[assign-name]`.","ref":"Mix.Tasks.Compile.Surface.html#module-options-for-other-types"},{"type":"task","title":"mix compile.surface.validate_components","doc":"","ref":"Mix.Tasks.Compile.Surface.ValidateComponents.html"},{"type":"function","title":"Mix.Tasks.Compile.Surface.ValidateComponents.validate/1","doc":"","ref":"Mix.Tasks.Compile.Surface.ValidateComponents.html#validate/1"},{"type":"task","title":"mix surface.convert","doc":"Converts .sface files and ~F sigils from v0.7 to v0.8 syntax.\n\n    mix surface.convert \"lib/**/*.{ex,exs,sface}\" \"test/**/*.{ex,exs}\"\n\nPlease read the [Migration Guide](https://github.com/surface-ui/surface/blob/main/MIGRATING.md)\nbefore running the converter and make sure you follow all required steps for a successful migration.","ref":"Mix.Tasks.Surface.Convert.html"},{"type":"task","title":"Task-specific options - mix surface.convert","doc":"The Surface formatter accepts a couple of the same task-specific options as `mix format`.\nHere are some examples of using these options:\n\n```bash\n$ mix surface.convert --dot-formatter path/to/.formatter.exs\n```\n\nYou can also use the same syntax as `mix format` for specifying which files to\nconvert:\n\n```bash\n$ mix surface.convert path/to/file.ex \"lib/**/*.{ex,exs,sface}\" \"test/**/*.{ex,exs}\"\n```","ref":"Mix.Tasks.Surface.Convert.html#module-task-specific-options"},{"type":"task","title":"mix surface.format","doc":"> #### Formatter Plugin Preferred {: .tip}\n>\n> To format Surface code using Elixir 1.13 or later, use\n> `Surface.Formatter.Plugin`.\n\nFormats Surface `~F` sigils and `.sface` files in the given files and patterns.\n\n```bash\n$ mix surface.format \"lib/**/*.{ex,exs}\" \"test/**/*.{ex,exs}\"\n$ cat path/to/file.ex | mix surface.format -\n```\n\nTakes the same options as `mix format` except for `--check-equivalent`.","ref":"Mix.Tasks.Surface.Format.html"},{"type":"task","title":"Formatting options - mix surface.format","doc":"Like `mix format`, the Surface formatter reads a `.formatter.exs` file in the\ncurrent directory for formatter configuration. The Surface formatter accepts\nthe same options as `mix format`. Read more about the expected format of\n`.formatter.exs` and the shared configuration options\n[documented here](https://hexdocs.pm/mix/main/Mix.Tasks.Format.html#module-formatting-options).\n\nThe Surface formatter also takes the following two additional options\nspecified in `.formatter.exs`:\n\n  - `:surface_line_length` overrides `:line_length` only for `mix surface.format`\n    (`:line_length` is used otherwise, or defaults to 98)\n  - `:surface_inputs` overrides `:inputs` only for `mix surface.format`\n    (`:inputs` is used otherwise)","ref":"Mix.Tasks.Surface.Format.html#module-formatting-options"},{"type":"task","title":"Task-specific options - mix surface.format","doc":"The Surface formatter accepts the same task-specific options as `mix format`.\n[Read documentation for the options documented here.](https://hexdocs.pm/mix/main/Mix.Tasks.Format.html#module-task-specific-options).\n\nFor quick reference, here are some examples of using these options:\n\n```bash\n$ mix surface.format --check-formatted\n** (Mix) mix surface.format failed due to --check-formatted.\nThe following files are not formatted:\n  * path/to/component.ex\n  * path/to/file.sface\n```\n\n```bash\n$ mix surface.format --dry-run\n```\n\n```bash\n$ mix surface.format --dot-formatter path/to/.formatter.exs\n```\n\nYou can also use the same syntax as `mix format` for specifying which files to\nformat:\n\n```bash\n$ mix surface.format path/to/file.ex \"lib/**/*.{ex,exs}\" \"test/**/*.{ex,exs}\"\n```","ref":"Mix.Tasks.Surface.Format.html#module-task-specific-options"},{"type":"task","title":"mix surface.init","doc":"Configures Surface on an phoenix project.\n\nThis task is mostly meant to be used after bootstrapping a fresh Phoenix project using\n`mix phx.new`. Although it can also be used on existing non-fresh projects, some patches\nmight not be applicable as they may depend on file content code/patterns that are no longer available.\n\nIf such cases are detected, the task will inform which patches were skipped as well as provide\ninstructions for manual configuration.\n\n    $ mix surface.init","ref":"Mix.Tasks.Surface.Init.html"},{"type":"task","title":"Important note - mix surface.init","doc":"This task is still **experimental**. Make sure you have committed your work or have a proper\nbackup before running it. As it may change a few files in the project, it's recommended to\nhave a safe way to rollback the changes in case anything goes wrong.","ref":"Mix.Tasks.Surface.Init.html#module-important-note"},{"type":"task","title":"Options - mix surface.init","doc":"* `--catalogue` - Configures the experimental Surface Catalogue for the project, which\n    will be available at the `/catalogue` route. For more information\n    see: https://github.com/surface-ui/surface_catalogue.\n\n  * `--demo` - Generates a sample ` ` component. When used together with the `--catalogue`\n    option, it additionally generates two catalogue examples and a playground for the component.\n\n  * `--layouts` - Replaces the generated `.heex` layouts with `.sface` files compatible with Surface.\n    Warning: using this option always replaces the existing layouts, so any changes done\n    in those layout files will be lost. It's recommended to use this option only on fresh projects.\n\n  * `--yes` - automatic answer \"yes\" to all prompts. Warning: this will also say \"yes\"\n    to overwrite existing files as well as fetching/installing dependencies, if required.\n\n  * `--dry-run` - does not save, delete or patch any file.\n\n  * `--no-formatter` - do not configure the Surface formatter.\n\n  * `--no-js-hooks` - do not configure automatic loading of colocated JS hook files.\n\n  * `--no-scoped-css` - do not configure scoped CSS for components.\n\n  * `--no-install` - do not fetch and install added/updated dependencies.\n\n  * `--web-module` - Configures the name of the Web module. Defaults to `MyAppWeb`.","ref":"Mix.Tasks.Surface.Init.html#module-options"},{"type":"module","title":"Surface","doc":"Surface is a component based library for **Phoenix LiveView**.\n\nBuilt on top of the new `Phoenix.LiveComponent` API, Surface provides\na more declarative way to express and use components in Phoenix.\n\nFull documentation and live examples can be found at [surface-ui.org](https://surface-ui.org)\n\nThis module defines the `~F` sigil that should be used to translate Surface\ncode into Phoenix templates.\n\nIn order to have `~F` available for any Phoenix view, add the following import to your web\nfile in `lib/my_app_web.ex`:\n\n    # lib/my_app_web.ex\n\n    ...\n\n    def view do\n      quote do\n        ...\n        import Surface\n      end\n    end\n\nAdditionally, use `Surface.init/1` in your mount function to initialize assigns used internally by surface:\n\n    # A LiveView using surface templates\n\n    defmodule PageLive do\n      use Phoenix.LiveView\n      import Surface\n\n      def mount(_params, _session, socket) do\n        socket = Surface.init(socket)\n        ...\n        {:ok, socket}\n      end\n\n      def render(assigns) do\n        ~F\"\"\"\n        ...\n        \"\"\"\n      end\n    end\n\n    # A LiveComponent using surface templates\n\n    defmodule NavComponent do\n      use Phoenix.LiveComponent\n      import Surface\n\n      def mount(socket) do\n        socket = Surface.init(socket)\n        ...\n        {:ok, socket}\n      end\n\n      def render(assigns) do\n        ~F\"\"\"\n        ...\n        \"\"\"\n      end\n    end","ref":"Surface.html"},{"type":"module","title":"Defining components - Surface","doc":"To create a component you need to define a module and `use` one of the available component types:\n\n  * `Surface.Component` - A stateless component.\n  * `Surface.LiveComponent` - A live stateful component.\n  * `Surface.LiveView` - A wrapper component around `Phoenix.LiveView`.\n  * `Surface.MacroComponent` - A low-level component which is responsible for translating its own content at compile time.","ref":"Surface.html#module-defining-components"},{"type":"module","title":"Example - Surface","doc":"# A functional stateless component\n\n    defmodule Button do\n      use Surface.Component\n\n      prop click, :event\n      prop kind, :string, default: \"is-info\"\n\n      def render(assigns) do\n        ~F\"\"\"\n         \n          <#slot/>\n         \n        \"\"\"\n      end\n    end\n\nYou can visit the documentation of each type of component for further explanation and examples.","ref":"Surface.html#module-example"},{"type":"macro","title":"Surface.embed_sface/1","doc":"Embeds an `.sface` template as a function component.","ref":"Surface.html#embed_sface/1"},{"type":"macro","title":"Example - Surface.embed_sface/1","doc":"defmodule MyAppWeb.Layouts do\n      use MyAppWeb, :html\n\n      embed_sface \"layouts/root.sface\"\n      embed_sface \"layouts/app.sface\"\n    end\n\nThe code above generates two functions, `root` and `app`. You can use both\nas regular function components or as layout templates.","ref":"Surface.html#embed_sface/1-example"},{"type":"function","title":"Surface.event_to_opts/2","doc":"","ref":"Surface.html#event_to_opts/2"},{"type":"function","title":"Surface.get_components_config/0","doc":"Retrieve all component's config","ref":"Surface.html#get_components_config/0"},{"type":"macro","title":"Surface.get_config/1","doc":"Retrieve the component's config based on the `key`","ref":"Surface.html#get_config/1"},{"type":"function","title":"Surface.get_config/2","doc":"Retrieve a component's config based on the `key`","ref":"Surface.html#get_config/2"},{"type":"function","title":"Surface.init/1","doc":"Initialize surface state in the socket","ref":"Surface.html#init/1"},{"type":"macro","title":"Surface.quote_surface/2","doc":"Converts the given code into Surface's AST.\n\nThe code must be passed with the `do` block using the `~F` sigil.\n\nOptional `line`, `file` and `caller` metadata can be passed using `opts`.","ref":"Surface.html#quote_surface/2"},{"type":"macro","title":"Example - Surface.quote_surface/2","doc":"iex> [tag] =\n    ...>   quote_surface do\n    ...>     ~F\" content \"\n    ...>   end\n    ...>\n    ...> tag.children\n    [%Surface.AST.Literal{directives: [], value: \"content\"}]","ref":"Surface.html#quote_surface/2-example"},{"type":"macro","title":"Surface.sigil_F/2","doc":"Translates Surface code into Phoenix templates.","ref":"Surface.html#sigil_F/2"},{"type":"macro","title":"Surface.slot_assigned?/1","doc":"Tests if a slot has been filled in.\n\nUseful to avoid rendering unnecessary html tags that are used to wrap an optional slot\nin combination with `:if` directive.","ref":"Surface.html#slot_assigned?/1"},{"type":"macro","title":"Examples - Surface.slot_assigned?/1","doc":"```\n   \n    <#slot {@header}/>\n   \n  ```","ref":"Surface.html#slot_assigned?/1-examples"},{"type":"behaviour","title":"Surface.Component","doc":"Defines a stateless component.","ref":"Surface.Component.html"},{"type":"behaviour","title":"Example - Surface.Component","doc":"defmodule Button do\n      use Surface.Component\n\n      prop click, :event\n\n      def render(assigns) do\n        ~F\"\"\"\n         \n          <#slot/>\n         \n        \"\"\"\n      end\n    end\n\n> **Note**: Stateless components cannot handle Phoenix LiveView events.\nIf you need to handle them, please use a `Surface.LiveComponent` instead.","ref":"Surface.Component.html#module-example"},{"type":"callback","title":"Surface.Component.render/1","doc":"","ref":"Surface.Component.html#c:render/1"},{"type":"module","title":"Surface.Formatter.Plugin","doc":"An [Elixir formatter\nplugin](https://hexdocs.pm/mix/1.13.0-rc.1/Mix.Tasks.Format.html#module-plugins)\nfor Surface code.\n\nElixir 1.13 introduced formatter plugins, allowing the Surface formatter to\nrun during `mix format` instead of requiring developers to run `mix\nsurface.format` separately.\n\nTo format Surface code using Elixir 1.12 or earlier, use `mix\nsurface.format`.\n\n### `.formatter.exs` setup\n\nAdd to `:plugins` in `.formatter.exs` in order to format `~F` sigils and\n`.sface` files when running `mix format`.\n\nOnly works on files matching patterns in `:inputs`, so add patterns for\nall Surface files to ensure they're formatted.\n\n    # in .formatter.exs\n    [\n      ...,\n      import_deps: [:surface],\n      plugins: [Surface.Formatter.Plugin],\n\n      # add patterns matching all .sface files and all .ex files with ~F sigils\n      inputs: [\"*.{ex,exs}\", \"{config,lib,test}/**/*.{ex,exs,sface}\"],\n\n      # THE FOLLOWING ARE OPTIONAL:\n\n      # set desired line length for both Elixir's code formatter and this one\n      # (only affects opening tags in Surface)\n      line_length: 80,\n\n      # or, set line length only for Surface code (overrides `line_length`)\n      surface_line_length: 84\n    ]","ref":"Surface.Formatter.Plugin.html"},{"type":"module","title":"Options - Surface.Formatter.Plugin","doc":"In `.formatter.exs`, the following options can be provided:\n\n- `:line_length` - Maximum line length of an opening tag before\n  SurfaceFormatter attempts to wrap it onto multiple lines. This option is\n  used by `Code.format_string!/2` and `mix format` and defaults to 98.\n- `:surface_line_length` - Overrides `:line_length`; useful for setting\n  separate desired line length for Surface code and non-Surface Elixir code.","ref":"Surface.Formatter.Plugin.html#module-options"},{"type":"function","title":"Surface.Formatter.Plugin.features/1","doc":"","ref":"Surface.Formatter.Plugin.html#features/1"},{"type":"function","title":"Surface.Formatter.Plugin.format/2","doc":"","ref":"Surface.Formatter.Plugin.html#format/2"},{"type":"module","title":"Surface.LiveComponent","doc":"A live stateful component. A wrapper around `Phoenix.LiveComponent`.","ref":"Surface.LiveComponent.html"},{"type":"module","title":"Example - Surface.LiveComponent","doc":"defmodule Dialog do\n      use Surface.LiveComponent\n\n      prop title, :string, required: true\n\n      def mount(socket) do\n        {:ok, assign(socket, show: false)}\n      end\n\n      def render(assigns) do\n        ~F\"\"\"\n         \n            \n           \n             \n               {@title} \n             \n             \n              <#slot/>\n             \n             \n               Ok \n             \n           \n         \n        \"\"\"\n      end\n\n      # Public API\n\n      def show(dialog_id) do\n        send_update(__MODULE__, id: dialog_id, show: true)\n      end\n\n      # Event handlers\n\n      def handle_event(\"show\", _, socket) do\n        {:noreply, assign(socket, show: true)}\n      end\n\n      def handle_event(\"hide\", _, socket) do\n        {:noreply, assign(socket, show: false)}\n      end\n    end","ref":"Surface.LiveComponent.html#module-example"},{"type":"module","title":"Surface.LiveView","doc":"A wrapper component around `Phoenix.LiveView`.\n\nSince this module is just a wrapper around `Phoenix.LiveView`, you\ncannot define custom properties for it. Only `:id` and `:session`\nare available. However, built-in directives like `:for` and `:if`\ncan be used normally.","ref":"Surface.LiveView.html"},{"type":"module","title":"Example - Surface.LiveView","doc":"defmodule Example do\n      use Surface.LiveView\n\n      def render(assigns) do\n        ~F\"\"\"\n         \n          This  Dialog  is a stateful component. Cool!\n         \n\n         Click to open the dialog \n        \"\"\"\n      end\n\n      def handle_event(\"show_dialog\", _, socket) do\n        Dialog.show(\"dialog\")\n        {:noreply, socket}\n      end\n    end","ref":"Surface.LiveView.html#module-example"},{"type":"module","title":"Surface.LiveViewTest","doc":"Conveniences for testing Surface components.","ref":"Surface.LiveViewTest.html"},{"type":"macro","title":"Surface.LiveViewTest.catalogue_test/2","doc":"This macro generates ExUnit test cases for catalogue examples.\n\nThe tests will automatically assert if the example was successfully rendered.\n\nPay attention that, by default, the generated tests don't test how the components should look like.\nHowever, it makes sure the examples are not raising exceptions at runtime, for instance, due to\nchanges in the component's API.","ref":"Surface.LiveViewTest.html#catalogue_test/2"},{"type":"macro","title":"Usage - Surface.LiveViewTest.catalogue_test/2","doc":"The `catalogue_test/1` macro accepts a single argument which can one of:\n\n* A component module (subject), which will generate tests for all examples/playgrounds found\n  for that component.\n* The atom `:all`, which will generate tests for all examples/playgrounds found for ALL components\n  in the project.\n\nKeep in mind that you should either use individual `catalogue_test/1` calls for each\ncomponent or use `:all`. Otherwise, you will end up with duplicated tests.","ref":"Surface.LiveViewTest.html#catalogue_test/2-usage"},{"type":"macro","title":"Options - Surface.LiveViewTest.catalogue_test/2","doc":"* `except` - A list of modules that should be excluded. This option only applies when using `:all`.","ref":"Surface.LiveViewTest.html#catalogue_test/2-options"},{"type":"macro","title":"Examples - Surface.LiveViewTest.catalogue_test/2","doc":"Generating tests for components' examples:\n\n    defmodule MyProject.Components.ButtonTest do\n      use MyProject.ConnCase, async: true\n\n      catalogue_test MyProject.Components.Button\n    end\n\nGenerating tests for all avaiable components:\n\n    defmodule MyProject.Components.CatalogueTest do\n      use MyProject.ConnCase, async: true\n\n      catalogue_test :all\n    end\n\nGenerating tests for all avaiable components except `MyComponent`:\n\n    defmodule MyProject.Components.CatalogueTest do\n      use MyProject.ConnCase, async: true\n\n      catalogue_test :all, except: [MyComponent]\n    end","ref":"Surface.LiveViewTest.html#catalogue_test/2-examples"},{"type":"macro","title":"Surface.LiveViewTest.compile_surface/2","doc":"Compiles Surface code into a new LiveView module.\n\nThis macro should be used sparingly as it always generates and compiles a new module\non-the-fly, which can potentially slow down your test suite.\n\nIts main use is when testing compile-time errors/warnings.","ref":"Surface.LiveViewTest.html#compile_surface/2"},{"type":"macro","title":"Example - Surface.LiveViewTest.compile_surface/2","doc":"code =\n      quote do\n        ~F\"\"\"\n         \n        \"\"\"\n      end\n\n    message =\n      ~S(code:1: invalid value for property \"prop\". Expected a :keyword, got: \"some string\".)\n\n    assert_raise(CompileError, message, fn ->\n      compile_surface(code)\n    end)","ref":"Surface.LiveViewTest.html#compile_surface/2-example"},{"type":"function","title":"Surface.LiveViewTest.register_propagate_context_to_slots/1","doc":"Registers components that propagates context into its slots.","ref":"Surface.LiveViewTest.html#register_propagate_context_to_slots/1"},{"type":"macro","title":"Surface.LiveViewTest.render_surface/1","doc":"Render Surface code.\n\nUse this macro when testing regular rendering of stateless components or live components\nthat don't require a parent live view during the tests.\n\nFor tests depending on the existence of a parent live view, e.g. testing events on live\ncomponents and its side-effects, you need to use either `Phoenix.LiveViewTest.live/2` or\n`Phoenix.LiveViewTest.live_isolated/3`.","ref":"Surface.LiveViewTest.html#render_surface/1"},{"type":"macro","title":"Example - Surface.LiveViewTest.render_surface/1","doc":"html =\n      render_surface do\n        ~F\"\"\"\n         \n        \"\"\"\n      end\n\n    assert html =~ \"\"\"\n           user \n          \"\"\"","ref":"Surface.LiveViewTest.html#render_surface/1-example"},{"type":"macro","title":"Surface.LiveViewTest.using_config/3","doc":"Wraps a test code so it runs using a custom configuration for a given component.\n\nTests using this macro should run synchronously. A warning is shown if the test\ncase is configured as `async: true`.","ref":"Surface.LiveViewTest.html#using_config/3"},{"type":"macro","title":"Example - Surface.LiveViewTest.using_config/3","doc":"using_config TextInput, default_class: \"default_class\" do\n      html =\n        render_surface do\n          ~F\"\"\"\n           \n          \"\"\"\n        end\n\n      assert html =~ ~r/class=\"default_class\"/\n    end","ref":"Surface.LiveViewTest.html#using_config/3-example"},{"type":"behaviour","title":"Surface.MacroComponent","doc":"A low-level component which is responsible for translating its own\ncontent at compile time.","ref":"Surface.MacroComponent.html"},{"type":"function","title":"Surface.MacroComponent.eval_static_props!/3","doc":"Evaluates the values of the static properties of a macro component.\n\nUsually called inside `translate/2` in order to retrieve the\nproperties' values at compile-time.","ref":"Surface.MacroComponent.html#eval_static_props!/3"},{"type":"callback","title":"Surface.MacroComponent.expand/3","doc":"This function is called to expand a macro component into a\nset of Surface.AST nodes.","ref":"Surface.MacroComponent.html#c:expand/3"},{"type":"module","title":"Surface.View","doc":"Use this module on regular Phoenix views to enable .sface templates","ref":"Surface.View.html"},{"type":"module","title":"Examples - Surface.View","doc":"To make sface templates available on all your views, add to the `view` function on `lib/your_app_web.ex`:\n\n    defmodule YourAppWeb do\n      # ...\n\n      def view do\n        use Phoenix.View,\n          root: \"lib/your_app_web/templates\",\n          namespace: YourAppWeb\n          \n        # ...\n        \n        use Surface.View, root: \"lib/your_app_web/templates\"\n      end\n    end\n\nIf you want to make it available only on specific views, you can add a new function on the `lib/your_app_web.ex`:\n\n    defmodule YourAppWeb do\n      # ...\n\n      def surface_view(options \\ []) do\n        [\n          view(options),\n          quote do\n            use Surface.View, root: \"lib/your_app_web/templates\"\n          end\n        ]\n      end\n    end\n\nThen, replace `use YourAppWeb, :view` with `use YourAppWeb, :surface_view` on the views you want to enable sface\ntemplates.","ref":"Surface.View.html#module-examples"},{"type":"function","title":"Surface.View.hash/2","doc":"Returns the hash of all template paths for the given view.\nUsed by Surface to check if a given view requires recompilation when a new template is added.","ref":"Surface.View.html#hash/2"},{"type":"module","title":"Surface.Components.Context","doc":"> #### Deprecation warning {: .warning}\n>\n> Using this module as a component with ` ` has been deprecated. Support for\n> scope-aware context will be removed in `v0.13` due to the lack of built-in support for\n> the feature in Liveview itself, which leads to inefficient diff-tracking when using it.\n>\n> Global context related functions like `Context.put/3` and `Context.get/3`, as well as data/prop's\n> option `:from_context`, will be kept and recommended as their usage don't affect diff-tracking\n> negatively.\n\nA built-in module and component that allows users to set and retrieve values\nto/from the context.","ref":"Surface.Components.Context.html"},{"type":"module","title":"Properties - Surface.Components.Context","doc":"* **put** *:context_put, accumulate: true, default: []* - Puts a value into the context.","ref":"Surface.Components.Context.html#module-properties"},{"type":"module","title":"Usage - Surface.Components.Context","doc":"```\n \n  ...\n \n```\n\nWhere:\n\n  * `scope` - Optional. Is an atom representing the scope where the values will\n    be stored. If no scope is provided, the value is stored at the root of the\n    context map.\n\n  * `values`- A keyword list containing the key-value pairs that will be stored\n    in the context.","ref":"Surface.Components.Context.html#module-usage"},{"type":"module","title":"Examples - Surface.Components.Context","doc":"With scope:\n\n```\n \n  ...\n \n```\n\nWithout scope:\n\n```\n \n  ...\n \n```\n\n\n* **get** *:context_get, accumulate: true, default: []* - Retrieves a set of values from the context binding them to local variables.","ref":"Surface.Components.Context.html#module-examples"},{"type":"module","title":"Usage - Surface.Components.Context","doc":"```\n \n  ...\n \n```\n\nWhere:\n\n  * `scope` - Optional. Is an atom representing the scope where the values will\n    be stored. If no scope is provided, the value is stored at the root of the\n    context map.\n\n  * `bindings`- A keyword list of bindings that will be retrieved from the context\n    as local variables.","ref":"Surface.Components.Context.html#module-usage"},{"type":"module","title":"Examples - Surface.Components.Context","doc":"```\n \n   \n \n```","ref":"Surface.Components.Context.html#module-examples"},{"type":"module","title":"Slots - Surface.Components.Context","doc":"* **default, required: true** - The content of the ` `","ref":"Surface.Components.Context.html#module-slots"},{"type":"function","title":"Surface.Components.Context.copy_assign/3","doc":"Copies a value from the context directly into `socket_or_assigns`.","ref":"Surface.Components.Context.html#copy_assign/3"},{"type":"function","title":"Usage - Surface.Components.Context.copy_assign/3","doc":"# Simple key\n    Context.copy_assign(socket_or_assigns, key)\n\n    # Simple key with options\n    Context.copy_assign(socket_or_assigns, key, opts)\n\n    # Key with scope\n    Context.copy_assign(socket_or_assigns, {scope, key})\n\n    # Key with scope + options\n    Context.copy_assign(socket_or_assigns, {scope, key}, opts)","ref":"Surface.Components.Context.html#copy_assign/3-usage"},{"type":"function","title":"Options - Surface.Components.Context.copy_assign/3","doc":"* `:as` - Optional. The key to store the value. Default is the `key` without the scope.","ref":"Surface.Components.Context.html#copy_assign/3-options"},{"type":"function","title":"Surface.Components.Context.get/3","doc":"Retrieves a value from the context.","ref":"Surface.Components.Context.html#get/3"},{"type":"function","title":"Usage - Surface.Components.Context.get/3","doc":"```\nContext.get(socket_or_assigns, scope, key)\n```\n\nWhere:\n\n  * `socket_or_assigns` - The socket or assigns where the values will be retrieved from.\n\n  * `scope` - Optional. Is an atom representing the scope where the values will\n    be stored. If no scope is provided, the value is stored at the root of the\n    context map.\n\n  * `key`- the key to look for when retrieving the value.","ref":"Surface.Components.Context.html#get/3-usage"},{"type":"function","title":"Examples - Surface.Components.Context.get/3","doc":"form = Context.get(assigns, Form, :form)\n    field = Context.get(assigns, Field, :field)\n\n    ...\n\n    ~F\"\"\"\n     \n    \"\"\"\n\n> **Note**: Whenever using `Context.get/3` inside the [`update/2`](`c:Phoenix.LiveComponent.update/2`) callback, make sure you call it passing the `socket`, not the `assigns`.","ref":"Surface.Components.Context.html#get/3-examples"},{"type":"function","title":"Surface.Components.Context.has_vars?/1","doc":"","ref":"Surface.Components.Context.html#has_vars?/1"},{"type":"function","title":"Surface.Components.Context.maybe_copy_assign/3","doc":"Copies a value from the context directly into `socket_or_assigns`\nif the value hasn't already been set or if it's `nil`.\n\nThe value will be saved using the same `key`.","ref":"Surface.Components.Context.html#maybe_copy_assign/3"},{"type":"function","title":"Usage - Surface.Components.Context.maybe_copy_assign/3","doc":"Context.maybe_copy_assign(socket_or_assigns, scope, key)","ref":"Surface.Components.Context.html#maybe_copy_assign/3-usage"},{"type":"function","title":"Surface.Components.Context.maybe_copy_assign!/3","doc":"Copies a value from the context directly into `socket_or_assigns`.\n\nIt raises a runtime error in case the value is still `nil` after\nafter the operation. This is useful whenever you expect a non-nil\nvalue coming either explicitly passed a `prop` or inplictly through\nthe context.\n\nThe value will be saved using the same `key`.","ref":"Surface.Components.Context.html#maybe_copy_assign!/3"},{"type":"function","title":"Usage - Surface.Components.Context.maybe_copy_assign!/3","doc":"Context.maybe_copy_assign!(socket_or_assigns, scope, key)","ref":"Surface.Components.Context.html#maybe_copy_assign!/3-usage"},{"type":"function","title":"Surface.Components.Context.put/3","doc":"Puts a value into the context.","ref":"Surface.Components.Context.html#put/3"},{"type":"function","title":"Usage - Surface.Components.Context.put/3","doc":"Context.put(socket_or_assigns, scope, values)\n\nWhere:\n\n  * `socket_or_assigns` - The socket or assigns where the values will be stored.\n\n  * `scope` - Optional. Is an atom representing the scope where the values will\n    be stored. If no scope is provided, the value is stored at the root of the\n    context map.\n\n  * `values`- A keyword list containing the key-value pairs that will be stored\n    in the context.","ref":"Surface.Components.Context.html#put/3-usage"},{"type":"function","title":"Examples - Surface.Components.Context.put/3","doc":"With scope:\n\n    Context.put(__MODULE__, form: @form)\n\nWithout scope:\n\n    Context.put(key1: @value1, key2: \"some other value\")\n\n> **Note**: Whenever using `Context.put/3` inside the [`update/2`](`c:Phoenix.LiveComponent.update/2`) callback, make sure you call it passing the `socket`, not the `assigns`.","ref":"Surface.Components.Context.html#put/3-examples"},{"type":"module","title":"Surface.Components.Dynamic.Component","doc":"A built-in component that allows users to inject dynamic components into\na Surface template.","ref":"Surface.Components.Dynamic.Component.html"},{"type":"module","title":"Properties - Surface.Components.Dynamic.Component","doc":"* **module** *:module, required: true* - The module of the component\n\n* **function** *:atom* - The function of the component","ref":"Surface.Components.Dynamic.Component.html#module-properties"},{"type":"module","title":"Slots - Surface.Components.Dynamic.Component","doc":"* **default** - The default slot","ref":"Surface.Components.Dynamic.Component.html#module-slots"},{"type":"function","title":"Surface.Components.Dynamic.Component.render/1","doc":"","ref":"Surface.Components.Dynamic.Component.html#render/1"},{"type":"module","title":"Surface.Components.Dynamic.LiveComponent","doc":"A built-in component that allows users to inject dynamic live components into\na Surface template.","ref":"Surface.Components.Dynamic.LiveComponent.html"},{"type":"module","title":"Properties - Surface.Components.Dynamic.LiveComponent","doc":"* **id** *:string, required: true* - The id of the live component (required by LiveView for stateful components).\n\n* **module** *:module, required: true* - The module of the component","ref":"Surface.Components.Dynamic.LiveComponent.html#module-properties"},{"type":"module","title":"Slots - Surface.Components.Dynamic.LiveComponent","doc":"* **default** - The default slot","ref":"Surface.Components.Dynamic.LiveComponent.html#module-slots"},{"type":"function","title":"Surface.Components.Dynamic.LiveComponent.mount/1","doc":"","ref":"Surface.Components.Dynamic.LiveComponent.html#mount/1"},{"type":"function","title":"Surface.Components.Dynamic.LiveComponent.render/1","doc":"","ref":"Surface.Components.Dynamic.LiveComponent.html#render/1"},{"type":"function","title":"Surface.Components.Dynamic.LiveComponent.update/2","doc":"","ref":"Surface.Components.Dynamic.LiveComponent.html#update/2"},{"type":"module","title":"Surface.Components.Link","doc":"> #### Deprecation warning {: .warning}\n>\n> This component has been deprecated in favor of liveview's built-in `<.link>`\n> and will be removed in `v0.13`. See https://hexdocs.pm/phoenix_live_view/live-navigation.html for\n> more info and usage.\n\nGenerates a link to the given URL.\n\nProvides similar capabilities to Phoenix's built-in `link/2` function.\n\nOptions `label` and `class` can be set directly and will override anything in `opts`.\nAll other options are forwarded to the underlying   tag.","ref":"Surface.Components.Link.html"},{"type":"module","title":"Examples - Surface.Components.Link","doc":"```\n \n\n \n   user \n \n```","ref":"Surface.Components.Link.html#module-examples"},{"type":"module","title":"Properties - Surface.Components.Link","doc":"* **to** *:any, required: true* - The page to link to\n* **method** *:atom, default: :get* - The method to use with the link\n* **id** *:string* - Id to apply to the link\n* **class** *:css_class* - Class or classes to apply to the link\n* **label** *:string* - The label for the generated ` ` element, if no content (default slot) is provided.\n\n* **opts** *:keyword, default: []* - Additional attributes to add onto the generated element\n\n* **values** *:keyword, default: []* - List values that will be sent as part of the payload triggered by an event","ref":"Surface.Components.Link.html#module-properties"},{"type":"module","title":"Slots - Surface.Components.Link","doc":"* **default** - The content of the generated ` ` element. If no content is provided,\nthe value of property `label` is used instead.","ref":"Surface.Components.Link.html#module-slots"},{"type":"module","title":"Events - Surface.Components.Link","doc":"* **click** - Triggered when the component receives click\n* **click_away** - Triggered when a click event happens outside of the element\n* **capture_click** - Triggered when the component captures click\n* **blur** - Triggered when the component loses focus\n* **focus** - Triggered when the component receives focus\n* **window_blur** - Triggered when the page loses focus\n* **window_focus** - Triggered when the page receives focus\n* **keydown** - Triggered when a key on the keyboard is pressed\n* **keyup** - Triggered when a key on the keyboard is released\n* **window_keydown** - Triggered when a key on the keyboard is pressed (window-level)\n* **window_keyup** - Triggered when a key on the keyboard is released (window-level)","ref":"Surface.Components.Link.html#module-events"},{"type":"function","title":"Surface.Components.Link.csrf_data/2","doc":"","ref":"Surface.Components.Link.html#csrf_data/2"},{"type":"function","title":"Surface.Components.Link.events_to_opts/1","doc":"","ref":"Surface.Components.Link.html#events_to_opts/1"},{"type":"function","title":"Surface.Components.Link.render/1","doc":"","ref":"Surface.Components.Link.html#render/1"},{"type":"function","title":"Surface.Components.Link.skip_csrf/1","doc":"","ref":"Surface.Components.Link.html#skip_csrf/1"},{"type":"function","title":"Surface.Components.Link.valid_destination!/2","doc":"","ref":"Surface.Components.Link.html#valid_destination!/2"},{"type":"function","title":"Surface.Components.Link.valid_string_destination!/2","doc":"","ref":"Surface.Components.Link.html#valid_string_destination!/2"},{"type":"module","title":"Surface.Components.LivePatch","doc":"> #### Deprecation warning {: .warning}\n>\n> This component has been deprecated in favor of liveview's built-in `<.link>`\n> and will be removed in `v0.13`. See https://hexdocs.pm/phoenix_live_view/live-navigation.html for\n> more info and usage.\n\nDefines a link that will **patch** the current LiveView.\n\nProvides similar capabilities to Phoenix's built-in `live_patch/2`\nfunction.\n\nWhen navigating to the current LiveView, `handle_params/3` is\nimmediately invoked to handle the change of params and URL state.\nThen the new state is pushed to the client, without reloading the\nwhole page. For live redirects to another LiveView, use\n` ` instead.","ref":"Surface.Components.LivePatch.html"},{"type":"module","title":"Properties - Surface.Components.LivePatch","doc":"* **to** *:string, required: true* - The required path to link to\n* **replace** *:boolean, default: false* - The flag to replace the current history or push a new state\n* **class** *:css_class, default: \"\"* - The CSS class for the generated ` ` element\n* **label** *:string* - The label for the generated ` ` element, if no content (default slot) is provided.\n\n* **opts** *:keyword, default: []* - Additional attributes to add onto the generated element","ref":"Surface.Components.LivePatch.html#module-properties"},{"type":"module","title":"Slots - Surface.Components.LivePatch","doc":"* **default** - The content of the generated ` ` element. If no content is provided,\nthe value of property `label` is used instead.","ref":"Surface.Components.LivePatch.html#module-slots"},{"type":"function","title":"Surface.Components.LivePatch.render/1","doc":"","ref":"Surface.Components.LivePatch.html#render/1"},{"type":"module","title":"Surface.Components.LiveRedirect","doc":"> #### Deprecation warning {: .warning}\n>\n> This component has been deprecated in favor of liveview's built-in `<.link>`\n> and will be removed in `v0.13`. See https://hexdocs.pm/phoenix_live_view/live-navigation.html for\n> more info and usage.\n\nDefines a link that will **redirect** to a new LiveView.\n\nProvides similar capabilities to Phoenix's built-in `live_redirect/2`\nfunction.\n\nThe current LiveView will be shut down and a new one will be mounted\nin its place, without reloading the whole page. This can\nalso be used to remount the same LiveView, in case you want to start\nfresh. If you want to navigate to the same LiveView without remounting\nit, use ` ` instead.","ref":"Surface.Components.LiveRedirect.html"},{"type":"module","title":"Properties - Surface.Components.LiveRedirect","doc":"* **to** *:string, required: true* - The required path to link to\n* **replace** *:boolean, default: false* - The flag to replace the current history or push a new state\n* **class** *:css_class, default: \"\"* - The CSS class for the generated ` ` element\n* **label** *:string* - The label for the generated ` ` element, if no content (default slot) is provided.\n\n* **opts** *:keyword, default: []* - Additional attributes to add onto the generated element","ref":"Surface.Components.LiveRedirect.html#module-properties"},{"type":"module","title":"Slots - Surface.Components.LiveRedirect","doc":"* **default** - The content of the generated ` ` element. If no content is provided,\nthe value of property `label` is used instead.","ref":"Surface.Components.LiveRedirect.html#module-slots"},{"type":"function","title":"Surface.Components.LiveRedirect.render/1","doc":"","ref":"Surface.Components.LiveRedirect.html#render/1"},{"type":"module","title":"Surface.Components.Raw","doc":"A macro component that does not translate any of its contents.\n\nThe content will be passed untouched to the underlying Phoenix's\ntemplate engine.\n\n> **Note**: By skipping translation, all Surface specific features\nare automatically disabled, including code interpolation with `{...}`,\nsyntactic sugar for attributes and markup validation.","ref":"Surface.Components.Raw.html"},{"type":"module","title":"Slots - Surface.Components.Raw","doc":"* **default** - The content that will not be translated by Surface","ref":"Surface.Components.Raw.html#module-slots"},{"type":"function","title":"Surface.Components.Raw.expand/3","doc":"","ref":"Surface.Components.Raw.html#expand/3"},{"type":"behaviour","title":"Surface.Catalogue","doc":"A behaviour to provide additional information about the catalogue.\n\nOptional for local catalogues. Usually required if you want to share\nyour components as a library.","ref":"Surface.Catalogue.html"},{"type":"callback","title":"Surface.Catalogue.config/0","doc":"Returns a keyword list of config options to be used by the catalogue tool.\n\nAvailable options:\n\n  * `head_css` - CSS related content to be added to the ` ... ` section\n    of each example or playground.\n\n  * `head_js` - JS related content to be added to the ` ... ` section\n    of each example or playground.\n\n  * `example` - A keyword list of options to be applied for all examples\n    in the catalogue.\n\n  * `playground` - A keyword list of options to be applied for all playgrounds\n    in the catalogue.","ref":"Surface.Catalogue.html#c:config/0"},{"type":"macro","title":"Surface.Catalogue.load_asset/2","doc":"Loads a text file as module attribute so you can inject its content directly\nin `head_css` or `head_js` config options.\n\nUseful to avoid depending on external css or js code. The path should be relative\nto the caller's folder.\n\nAvailable options:\n\n  * `as` - the name of the module attribute to be generated.","ref":"Surface.Catalogue.html#load_asset/2"},{"type":"module","title":"Surface.Catalogue.Data","doc":"Experimental module that provides conveniences for manipulating data\nin Examples and Playgrounds.\n\nProvide wrappers around built-in functions like `get_in/2` and `update_in/3`\nusing a shorter syntax for accessors.","ref":"Surface.Catalogue.Data.html"},{"type":"module","title":"Accessor Mapping - Surface.Catalogue.Data","doc":"* `[_]`: `Access.all/0`\n  * `[fun]`: `Access.filter(fun)`\n  * `[index]`: `Access.at(index)` (Shorthand for `[index: i]`)\n  * `[index: i]`: `Access.at(i)`\n  * `[key: k]`: `Access.key(k)`\n  * `[first..last]`: `Surface.Catalogue.Data.slice(first..last)`","ref":"Surface.Catalogue.Data.html#module-accessor-mapping"},{"type":"module","title":"Example - Surface.Catalogue.Data","doc":"Data.get(props.lists[_].cards[& &1.id == \"Card_1\"].tags[-1].name)\n\nThe code above will be translated to:\n\n    get_in(props, [:lists, Access.all, :cards, Access.filter(& &1.id == \"Card_1\"), :tags, Access.at(-1), :name])","ref":"Surface.Catalogue.Data.html#module-example"},{"type":"macro","title":"Surface.Catalogue.Data.append/2","doc":"Appends an item to a list in the given nested structure.","ref":"Surface.Catalogue.Data.html#append/2"},{"type":"macro","title":"Surface.Catalogue.Data.delete/1","doc":"Deletes an item from the given nested structure.","ref":"Surface.Catalogue.Data.html#delete/1"},{"type":"macro","title":"Surface.Catalogue.Data.get/1","doc":"Gets a value from the given nested structure.\n\nA wrapper around `get_in/2`","ref":"Surface.Catalogue.Data.html#get/1"},{"type":"macro","title":"Surface.Catalogue.Data.get!/1","doc":"Gets an existing value from the given nested structure.\n\nRaises an error if none or more than one value is found.","ref":"Surface.Catalogue.Data.html#get!/1"},{"type":"macro","title":"Surface.Catalogue.Data.get_and_update/2","doc":"Gets a value and updates a given nested structure.\n\nA wrapper around `get_and_update_in/3`","ref":"Surface.Catalogue.Data.html#get_and_update/2"},{"type":"macro","title":"Surface.Catalogue.Data.insert_at/3","doc":"Inserts an item into a list in the given nested structure.","ref":"Surface.Catalogue.Data.html#insert_at/3"},{"type":"macro","title":"Surface.Catalogue.Data.pop/1","doc":"Pops a item from the given nested structure.\n\nA wrapper around `pop_in/2`","ref":"Surface.Catalogue.Data.html#pop/1"},{"type":"macro","title":"Surface.Catalogue.Data.prepend/2","doc":"Prepends an item to a list in the given nested structure.","ref":"Surface.Catalogue.Data.html#prepend/2"},{"type":"function","title":"Surface.Catalogue.Data.random_id/1","doc":"Generates a short ramdom id.","ref":"Surface.Catalogue.Data.html#random_id/1"},{"type":"function","title":"Surface.Catalogue.Data.slice/1","doc":"","ref":"Surface.Catalogue.Data.html#slice/1"},{"type":"function","title":"Surface.Catalogue.Data.unnest/1","doc":"","ref":"Surface.Catalogue.Data.html#unnest/1"},{"type":"macro","title":"Surface.Catalogue.Data.update/2","doc":"Updates an item in the given nested structure.\n\nA wrapper around `update_in/2`","ref":"Surface.Catalogue.Data.html#update/2"},{"type":"module","title":"Surface.Catalogue.Examples","doc":"A generic LiveView to create multiple stateless examples for the Surface Catalogue.\n\nEach example must be a function component defined with the module attribute `@example`.\n\n> **NOTE**: If your examples require manipulating state (data) through `handle_event` callbacks,\n> use `Surface.Catalogue.LiveExample` instead.","ref":"Surface.Catalogue.Examples.html"},{"type":"module","title":"Options - Surface.Catalogue.Examples","doc":"Besides the buit-in options provided by the LiveView itself, examples can also\npass the following options at the module level:\n\n  * `subject` - Required. The target component of the Example.\n\n  * `catalogue` - Optional. A module that implements the `Surface.Catalogue`\n    providing additional information to the catalogue tool. Usually required\n    if you want to share your components as a library.\n\nOther options can be defined at the module level but also overridden at the\nfunction/example level using the `@example` module attribute. They are:\n\n  * `height` - Required (either for the module or function). The height of the Example.\n\n  * `title` - Optional. The title of the example.\n\n  * `body` - Optional. Sets/overrides the attributes of the the Example's body tag.\n    Useful to set a different background or padding.\n\n  * `direction` - Optional. Defines how the example + code boxes should be displayed.\n    Available values are \"horizontal\" or \"vertical\". Default is \"horizontal\" (side-by-side).\n\n  * `code_perc` - Optional. When the direction is \"horizontal\", defines the percentage of\n    the total width that the code box should take. Default is `50`. Note: This configuration\n    has no effect when direction is \"vertical\".\n\n  * `assert` - Optional. When using `catalogue_test/1`, generates simple `=~` assertions for\n    the given text or list of texts.\n\nWhen defined at the module level, i.e. passing to `use Surface.Catalogue.Examples, ...`, the\noptions apply to all examples. Aside from `subject` and `catalogue`, options can be overridden\nat the function level using the `@example` module attribute.","ref":"Surface.Catalogue.Examples.html#module-options"},{"type":"module","title":"Examples - Surface.Catalogue.Examples","doc":"","ref":"Surface.Catalogue.Examples.html#module-examples"},{"type":"module","title":"Basic usage - Surface.Catalogue.Examples","doc":"```\n@example true\ndef basic_example(assigns) do\n  ~F\"\"\"\n   OK \n  \"\"\"\nend\n```","ref":"Surface.Catalogue.Examples.html#module-basic-usage"},{"type":"module","title":"Defining a custom title + docs - Surface.Catalogue.Examples","doc":"```\n@example \"Example with docs\"\n@doc \"A simple example with documentation\"\ndef example_01(assigns) do\n  ~F\"\"\"\n   OK \n  \"\"\"\nend\n```","ref":"Surface.Catalogue.Examples.html#module-defining-a-custom-title-docs"},{"type":"module","title":"Defining other options - Surface.Catalogue.Examples","doc":"```\n@example [\n  title: \"Example and Code split vertically\",\n  direction: \"vertical\",\n  height: \"110px\",\n  assert: [\"Small\", \"Medium\", \"Large\"]\n]\ndef vertical(assigns) do\n  ~F\"\"\"\n   Small \n   Medium \n   Large \n  \"\"\"\nend\n```\n\nNotice that whenever you want to pass other options using a keyword list as in the\nexample above, in case you need to customize the title, it must to be passed as\na key/value option as well.","ref":"Surface.Catalogue.Examples.html#module-defining-other-options"},{"type":"module","title":"Surface.Catalogue.LiveExample","doc":"A generic LiveView to create a single live example for the Surface Catalogue.\n\nUse `LiveExample` if your example requires manipulating state (data) through\n`handle_event` callbacks. For stateless examples, you should use `Surface.Catalogue.Examples`\nas it allows defining multiple examples on a single module.","ref":"Surface.Catalogue.LiveExample.html"},{"type":"module","title":"Options - Surface.Catalogue.LiveExample","doc":"Besides the buit-in options provided by the LiveView itself, a LiveExample also\nprovides the following options:\n\n  * `subject` - Required. The target component of the Example.\n\n  * `catalogue` - Optional. A module that implements the `Surface.Catalogue`\n    providing additional information to the catalogue tool. Usually required\n    if you want to share your components as a library.\n\n  * `height` - Required. The height of the Example.\n\n  * `title` - Optional. The title of the example.\n\n  * `body` - Optional. Sets/overrides the attributes of the the Example's body tag.\n    Useful to set a different background or padding.\n\n  * `direction` - Optional. Defines how the example + code boxes should be displayed.\n    Available values are \"horizontal\" or \"vertical\". Default is \"horizontal\" (side-by-side).\n\n  * `code_perc` - Optional. When the direction is \"horizontal\", defines the percentage of\n    the total width that the code box should take. Default is `50`. Note: This configuration\n    has no effect when direction is \"vertical\".\n\n  * `assert` - Optional. When using `catalogue_test/1`, generates simple `=~` assertions for\n    the given text or list of texts.","ref":"Surface.Catalogue.LiveExample.html#module-options"},{"type":"module","title":"Example - Surface.Catalogue.LiveExample","doc":"defmodule MyApp.Catalogue.MyComponentExample do\n      use Surface.Catalogue.LiveExample,\n        subject: MyApp.MyComponent,\n        title: \"An example for MyComponent\",\n        assert: [\"Value is 0\"]\n\n      data value, :integer, default: 0\n\n      def render(assigns) do\n        ~F\"\"\"\n         \n        \"\"\"\n      end\n    end","ref":"Surface.Catalogue.LiveExample.html#module-example"},{"type":"module","title":"Surface.Catalogue.Playground","doc":"A generic LiveView to create a component's playground for the Surface Catalogue.","ref":"Surface.Catalogue.Playground.html"},{"type":"module","title":"Options - Surface.Catalogue.Playground","doc":"Besides the buit-in options provided by the LiveView itself, a Playground also\nprovides the following options:\n\n  * `subject` - Required. The target component of the Playground.\n\n  * `height` - Required. The initial height of the Playground.\n\n  * `catalogue` - Optional. A module that implements the `Surface.Catalogue`\n    providing additional information to the catalogue tool. Usually required\n    if you want to share your components as a library.\n\n  * `body` - Optional. Sets/overrides the attributes of the Playground's body tag.\n    Useful to set a different background or padding.","ref":"Surface.Catalogue.Playground.html#module-options"},{"type":"module","title":"Initializing props and slots - Surface.Catalogue.Playground","doc":"If you need to define initial values for props or slots of the component, you can use\nthe `@props` and `@slots` module attributes, respectively. Both attributes are optional\nand take a keyword list containing all the props/slots you need to initialize.\n\nPay attention that if you have required props/slots, you should provide initial values\nfor them.","ref":"Surface.Catalogue.Playground.html#module-initializing-props-and-slots"},{"type":"module","title":"Example - Surface.Catalogue.Playground","doc":"defmodule MyApp.Components.MyButton.Playground do\n      use Surface.Catalogue.Playground,\n        subject: MyApp.Components.MyButton,\n        height: \"170px\"\n\n      @props [\n        class: \"btn\"\n      ]\n\n      @slots [\n        default: \"Cancel\"\n      ]\n    end","ref":"Surface.Catalogue.Playground.html#module-example"},{"type":"module","title":"Surface.Compiler","doc":"Defines a behaviour that must be implemented by all HTML/Surface node translators.\n\nThis module also contains the main logic to translate Surface code.","ref":"Surface.Compiler.html"},{"type":"function","title":"Surface.Compiler.compile/5","doc":"This function compiles a string into the Surface AST.This is used by ~F and Surface.Renderer to parse and compile templates.\n\nA special note for line: This is considered the line number for the first line in the string. If the first line of the\nstring is also the first line of the file, then this should be 1. If this is being called within a macro (say to process a heredoc\npassed to ~F), this should be __CALLER__.line + 1.","ref":"Surface.Compiler.html#compile/5"},{"type":"function","title":"Surface.Compiler.to_ast/2","doc":"","ref":"Surface.Compiler.html#to_ast/2"},{"type":"function","title":"Surface.Compiler.to_live_struct/2","doc":"","ref":"Surface.Compiler.html#to_live_struct/2"},{"type":"function","title":"Surface.Compiler.validate_component_structure/3","doc":"","ref":"Surface.Compiler.html#validate_component_structure/3"},{"type":"exception","title":"Surface.Compiler.CSSParserError","doc":"","ref":"Surface.Compiler.CSSParserError.html"},{"type":"module","title":"Surface.Compiler.CallerSpec","doc":"","ref":"Surface.Compiler.CallerSpec.html"},{"type":"type","title":"Surface.Compiler.CallerSpec.t/0","doc":"","ref":"Surface.Compiler.CallerSpec.html#t:t/0"},{"type":"module","title":"Surface.Compiler.CompileMeta","doc":"","ref":"Surface.Compiler.CompileMeta.html"},{"type":"type","title":"Surface.Compiler.CompileMeta.t/0","doc":"","ref":"Surface.Compiler.CompileMeta.html#t:t/0"},{"type":"module","title":"Surface.Compiler.EExEngine","doc":"This module glues together surface's AST and Phoenix.LiveView.Engine to actually render an AST.\n\nIt takes a list of Surface AST nodes, and processes them into a sequence of static raw html items and\ndynamic pieces. It then converts these into tokens which an EEx engine can understand (see EEx.Tokenizer\nfor information on this). Finally, it passes these tokens into the engine sequentially in the same\nmanner as EEx.Compiler.compile/2","ref":"Surface.Compiler.EExEngine.html"},{"type":"function","title":"Surface.Compiler.EExEngine.translate/2","doc":"","ref":"Surface.Compiler.EExEngine.html#translate/2"},{"type":"exception","title":"Surface.Compiler.ParseError","doc":"","ref":"Surface.Compiler.ParseError.html"},{"type":"behaviour","title":"Surface.Directive","doc":"","ref":"Surface.Directive.html"},{"type":"callback","title":"Surface.Directive.extract/2","doc":"","ref":"Surface.Directive.html#c:extract/2"},{"type":"callback","title":"Surface.Directive.process/2","doc":"","ref":"Surface.Directive.html#c:process/2"},{"type":"module","title":"Surface.Directive.ComponentProps","doc":"","ref":"Surface.Directive.ComponentProps.html"},{"type":"function","title":"Surface.Directive.ComponentProps.extract/2","doc":"","ref":"Surface.Directive.ComponentProps.html#extract/2"},{"type":"function","title":"Surface.Directive.ComponentProps.process/2","doc":"","ref":"Surface.Directive.ComponentProps.html#process/2"},{"type":"module","title":"Surface.Directive.Debug","doc":"","ref":"Surface.Directive.Debug.html"},{"type":"function","title":"Surface.Directive.Debug.extract/2","doc":"","ref":"Surface.Directive.Debug.html#extract/2"},{"type":"function","title":"Surface.Directive.Debug.process/2","doc":"","ref":"Surface.Directive.Debug.html#process/2"},{"type":"module","title":"Surface.Directive.Events","doc":"","ref":"Surface.Directive.Events.html"},{"type":"function","title":"Surface.Directive.Events.extract/2","doc":"","ref":"Surface.Directive.Events.html#extract/2"},{"type":"function","title":"Surface.Directive.Events.phx_events/0","doc":"","ref":"Surface.Directive.Events.html#phx_events/0"},{"type":"function","title":"Surface.Directive.Events.process/2","doc":"","ref":"Surface.Directive.Events.html#process/2"},{"type":"module","title":"Surface.Directive.For","doc":"","ref":"Surface.Directive.For.html"},{"type":"function","title":"Surface.Directive.For.extract/2","doc":"","ref":"Surface.Directive.For.html#extract/2"},{"type":"function","title":"Surface.Directive.For.process/2","doc":"","ref":"Surface.Directive.For.html#process/2"},{"type":"module","title":"Surface.Directive.Hook","doc":"","ref":"Surface.Directive.Hook.html"},{"type":"function","title":"Surface.Directive.Hook.extract/2","doc":"","ref":"Surface.Directive.Hook.html#extract/2"},{"type":"function","title":"Surface.Directive.Hook.process/2","doc":"","ref":"Surface.Directive.Hook.html#process/2"},{"type":"module","title":"Surface.Directive.If","doc":"","ref":"Surface.Directive.If.html"},{"type":"function","title":"Surface.Directive.If.extract/2","doc":"","ref":"Surface.Directive.If.html#extract/2"},{"type":"function","title":"Surface.Directive.If.process/2","doc":"","ref":"Surface.Directive.If.html#process/2"},{"type":"module","title":"Surface.Directive.Let","doc":"","ref":"Surface.Directive.Let.html"},{"type":"function","title":"Surface.Directive.Let.extract/2","doc":"","ref":"Surface.Directive.Let.html#extract/2"},{"type":"function","title":"Surface.Directive.Let.process/2","doc":"","ref":"Surface.Directive.Let.html#process/2"},{"type":"module","title":"Surface.Directive.Show","doc":"","ref":"Surface.Directive.Show.html"},{"type":"function","title":"Surface.Directive.Show.extract/2","doc":"","ref":"Surface.Directive.Show.html#extract/2"},{"type":"function","title":"Surface.Directive.Show.process/2","doc":"","ref":"Surface.Directive.Show.html#process/2"},{"type":"module","title":"Surface.Directive.TagAttrs","doc":"","ref":"Surface.Directive.TagAttrs.html"},{"type":"function","title":"Surface.Directive.TagAttrs.extract/2","doc":"","ref":"Surface.Directive.TagAttrs.html#extract/2"},{"type":"function","title":"Surface.Directive.TagAttrs.process/2","doc":"","ref":"Surface.Directive.TagAttrs.html#process/2"},{"type":"module","title":"Surface.Directive.Values","doc":"","ref":"Surface.Directive.Values.html"},{"type":"function","title":"Surface.Directive.Values.extract/2","doc":"","ref":"Surface.Directive.Values.html#extract/2"},{"type":"function","title":"Surface.Directive.Values.process/2","doc":"","ref":"Surface.Directive.Values.html#process/2"},{"type":"function","title":"Surface.Directive.Values.validate_value!/2","doc":"","ref":"Surface.Directive.Values.html#validate_value!/2"},{"type":"module","title":"Surface.AST","doc":"","ref":"Surface.AST.html"},{"type":"function","title":"Surface.AST.find_attribute_value/2","doc":"","ref":"Surface.AST.html#find_attribute_value/2"},{"type":"function","title":"Surface.AST.has_attribute?/2","doc":"","ref":"Surface.AST.html#has_attribute?/2"},{"type":"function","title":"Surface.AST.pop_attributes_as_map/2","doc":"","ref":"Surface.AST.html#pop_attributes_as_map/2"},{"type":"function","title":"Surface.AST.pop_attributes_values_as_map/2","doc":"","ref":"Surface.AST.html#pop_attributes_values_as_map/2"},{"type":"type","title":"Surface.AST.t/0","doc":"","ref":"Surface.AST.html#t:t/0"},{"type":"module","title":"Surface.AST.Attribute","doc":"An AST node representing an attribute or property","ref":"Surface.AST.Attribute.html"},{"type":"module","title":"Properties - Surface.AST.Attribute","doc":"* `:type` - an atom representing the type of attribute. See Surface.API for the list of supported types\n    * `:type_opts` - a keyword list of options for the type\n    * `:name` - the name of the attribute (e.g. `:class`)\n    * `:root` - true if the attribute was defined using root notation `{ ... }`. Root attributes won't have `name`.\n    * `:value` - a list of nodes that can be concatenated to form the value for this attribute. Potentially contains dynamic data\n    * `:meta` - compilation meta data","ref":"Surface.AST.Attribute.html#module-properties"},{"type":"type","title":"Surface.AST.Attribute.t/0","doc":"","ref":"Surface.AST.Attribute.html#t:t/0"},{"type":"module","title":"Surface.AST.AttributeExpr","doc":"An AST node representing an attribute expression (i.e. a dynamic value for an attribute, directive, or property)","ref":"Surface.AST.AttributeExpr.html"},{"type":"module","title":"Properties - Surface.AST.AttributeExpr","doc":"* `:original` - the original text, useful for debugging and error messages\n    * `:value` - a quoted expression\n    * `:constant?` - true if the expression can be evaluated at compile time\n    * `:meta` - compilation meta data","ref":"Surface.AST.AttributeExpr.html#module-properties"},{"type":"function","title":"Surface.AST.AttributeExpr.new/3","doc":"","ref":"Surface.AST.AttributeExpr.html#new/3"},{"type":"type","title":"Surface.AST.AttributeExpr.t/0","doc":"","ref":"Surface.AST.AttributeExpr.html#t:t/0"},{"type":"module","title":"Surface.AST.Block","doc":"An AST node representing a generic block.","ref":"Surface.AST.Block.html"},{"type":"module","title":"Properties - Surface.AST.Block","doc":"* `:name` - name of the block\n    * `:expression` - the expression passed to block\n    * `:sub_blocks` - a list containing each sub-block's {name, children_ast}\n    * `:meta` - compile meta\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.Block.html#module-properties"},{"type":"type","title":"Surface.AST.Block.t/0","doc":"","ref":"Surface.AST.Block.html#t:t/0"},{"type":"module","title":"Surface.AST.Component","doc":"An AST node representing a component","ref":"Surface.AST.Component.html"},{"type":"module","title":"Properties - Surface.AST.Component","doc":"* `:module` - the component module\n    * `:type` - the type of component (i.e. Surface.LiveComponent vs Surface.LiveView or :dynamic_live)\n    * `:props` - the props for this component\n    * `:directives` - any directives to be applied to this tag\n    * `:children` - the tag children\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.Component.html#module-properties"},{"type":"type","title":"Surface.AST.Component.t/0","doc":"","ref":"Surface.AST.Component.html#t:t/0"},{"type":"module","title":"Surface.AST.Container","doc":"An AST node representing a container of other nodes. This does not\nhave content itself, just contains children which have content, and\ndirectives that operate on the entirety of the children (i.e. for, if, debug)","ref":"Surface.AST.Container.html"},{"type":"module","title":"Properties - Surface.AST.Container","doc":"* `:children` - children AST nodes\n    * `:attributes` - the specified attributes\n    * `:directives` - directives associated with this container\n    * `:meta` - compile meta\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.Container.html#module-properties"},{"type":"type","title":"Surface.AST.Container.t/0","doc":"","ref":"Surface.AST.Container.html#t:t/0"},{"type":"module","title":"Surface.AST.Directive","doc":"An AST node representing a directive","ref":"Surface.AST.Directive.html"},{"type":"module","title":"Properties - Surface.AST.Directive","doc":"* `:module` - the module which implements logic for this directive (e.g. `Surface.Directive.Let`)\n    * `:name` - the name of the directive (e.g. `:let`)\n    * `:value` - the code/configuration for this directive. typically a quoted expression\n    * `:meta` - compilation meta data","ref":"Surface.AST.Directive.html#module-properties"},{"type":"type","title":"Surface.AST.Directive.t/0","doc":"","ref":"Surface.AST.Directive.html#t:t/0"},{"type":"module","title":"Surface.AST.DynamicAttribute","doc":"An AST node representing a dynamic attribute (or attributes).","ref":"Surface.AST.DynamicAttribute.html"},{"type":"module","title":"Properties - Surface.AST.DynamicAttribute","doc":"* `:expr` - an expression which will generate a keyword list of attributes and value tuples of the form {type, value}\n    * `:meta` - compilation meta data","ref":"Surface.AST.DynamicAttribute.html#module-properties"},{"type":"type","title":"Surface.AST.DynamicAttribute.t/0","doc":"","ref":"Surface.AST.DynamicAttribute.html#t:t/0"},{"type":"module","title":"Surface.AST.Error","doc":"An AST node representing an error. This will be rendered as an html element.","ref":"Surface.AST.Error.html"},{"type":"module","title":"Properties - Surface.AST.Error","doc":"* `:message` - the error message\n    * `:meta` - compilation meta data\n    * `:attributes` - the specified attributes\n    * `:directives` - directives associated with this error node","ref":"Surface.AST.Error.html#module-properties"},{"type":"type","title":"Surface.AST.Error.t/0","doc":"","ref":"Surface.AST.Error.html#t:t/0"},{"type":"module","title":"Surface.AST.Expr","doc":"An AST node representing an expression which does not resolve to a value printed out to the final DOM.","ref":"Surface.AST.Expr.html"},{"type":"module","title":"Properties - Surface.AST.Expr","doc":"* `:value` - a quoted expression\n    * `:constant?` - true if the expression can be evaluated at compile time\n    * `:meta` - compile meta\n    * `:directives` - directives associated with this expression node","ref":"Surface.AST.Expr.html#module-properties"},{"type":"type","title":"Surface.AST.Expr.t/0","doc":"","ref":"Surface.AST.Expr.html#t:t/0"},{"type":"module","title":"Surface.AST.For","doc":"An AST node representing a for comprehension.","ref":"Surface.AST.For.html"},{"type":"module","title":"Properties - Surface.AST.For","doc":"* `:generator` - a quoted expression\n    * `:children` - the children to collect over the generator\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation\n    * `:directives` - directives associated with this node","ref":"Surface.AST.For.html#module-properties"},{"type":"type","title":"Surface.AST.For.t/0","doc":"","ref":"Surface.AST.For.html#t:t/0"},{"type":"module","title":"Surface.AST.FunctionComponent","doc":"An AST node representing a function component","ref":"Surface.AST.FunctionComponent.html"},{"type":"module","title":"Properties - Surface.AST.FunctionComponent","doc":"* `:module` - the component module\n    * `:fun` - the render function\n    * `:type` - the type of function (:local or :remote)\n    * `:props` - the props for this component\n    * `:directives` - any directives to be applied to this tag\n    * `:children` - the tag children\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.FunctionComponent.html#module-properties"},{"type":"type","title":"Surface.AST.FunctionComponent.t/0","doc":"","ref":"Surface.AST.FunctionComponent.html#t:t/0"},{"type":"module","title":"Surface.AST.If","doc":"An AST node representing an if/else expression","ref":"Surface.AST.If.html"},{"type":"module","title":"Properties - Surface.AST.If","doc":"* `:condition` - a quoted expression\n    * `:children` - the children to insert into the dom if the condition evaluates truthy\n    * `:else` - the children to insert into the dom if the condition evaluates falsy\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation\n    * `:directives` - directives associated with this node","ref":"Surface.AST.If.html#module-properties"},{"type":"type","title":"Surface.AST.If.t/0","doc":"","ref":"Surface.AST.If.html#t:t/0"},{"type":"module","title":"Surface.AST.Interpolation","doc":"An AST node representing interpolation within a node","ref":"Surface.AST.Interpolation.html"},{"type":"module","title":"Properties - Surface.AST.Interpolation","doc":"* `:original` - the original text, useful for debugging and error messages\n    * `:value` - a quoted expression\n    * `:constant?` - true if the expression can be evaluated at compile time\n    * `:meta` - compilation meta data\n    * `:directives` - directives associated with this interpolation","ref":"Surface.AST.Interpolation.html#module-properties"},{"type":"type","title":"Surface.AST.Interpolation.t/0","doc":"","ref":"Surface.AST.Interpolation.html#t:t/0"},{"type":"module","title":"Surface.AST.Literal","doc":"An AST node representing a literal value","ref":"Surface.AST.Literal.html"},{"type":"module","title":"Properties - Surface.AST.Literal","doc":"* `:value` - the value\n    * `:directives` - directives associated with this literal value","ref":"Surface.AST.Literal.html#module-properties"},{"type":"type","title":"Surface.AST.Literal.t/0","doc":"","ref":"Surface.AST.Literal.html#t:t/0"},{"type":"module","title":"Surface.AST.MacroComponent","doc":"An AST node representing a macro component","ref":"Surface.AST.MacroComponent.html"},{"type":"module","title":"Properties - Surface.AST.MacroComponent","doc":"* `:module` - the component module\n    * `:attributes` - the specified attributes\n    * `:directives` - any directives to be applied to this macro\n    * `:children` - the children after the macro is expanded\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.MacroComponent.html#module-properties"},{"type":"type","title":"Surface.AST.MacroComponent.t/0","doc":"","ref":"Surface.AST.MacroComponent.html#t:t/0"},{"type":"module","title":"Surface.AST.Meta","doc":"A container for metadata about compilation.","ref":"Surface.AST.Meta.html"},{"type":"module","title":"Properties - Surface.AST.Meta","doc":"* `:line` - the line from the source code where the parent was extracted\n    * `:column` - the column from the source code where the parent was extracted\n    * `:module` - the component module (e.g. `Surface.Components.LivePatch`)\n    * `:node_alias` - the alias used inside the source code (e.g. `LivePatch`)\n    * `:file` - the file from which the source was extracted\n    * `:caller` - a Macro.Env struct representing the caller\n    * `:style` - the style info of the component, if any\n    * `:caller_spec` - the specs of the caller component","ref":"Surface.AST.Meta.html#module-properties"},{"type":"type","title":"Surface.AST.Meta.t/0","doc":"","ref":"Surface.AST.Meta.html#t:t/0"},{"type":"module","title":"Surface.AST.Slot","doc":"An AST node representing a <#slot /> tag","ref":"Surface.AST.Slot.html"},{"type":"module","title":"Properties - Surface.AST.Slot","doc":"* `:name` - the slot name\n    * `:for` - the slotable entry assigned for this slot\n    * `:default` - a list of AST nodes representing the default content for this slot\n    * `:arg` - quoted expression representing the argument for this slot\n    * `:generator_value` - value from the `:generator_prop` property\n    * `:context_put` - value from the `:generator_prop` property\n    * `:meta` - compilation meta data\n    * `:directives` - directives associated with this slot","ref":"Surface.AST.Slot.html#module-properties"},{"type":"type","title":"Surface.AST.Slot.t/0","doc":"","ref":"Surface.AST.Slot.html#t:t/0"},{"type":"module","title":"Surface.AST.SlotEntry","doc":"An AST node representing a <:slot> entry. This is used to provide content for slots","ref":"Surface.AST.SlotEntry.html"},{"type":"module","title":"Properties - Surface.AST.SlotEntry","doc":"* `:name` - the slot entry name\n    * `:props` - the props for slot entry tag\n    * `:let` - the `:let` expression\n    * `:children` - the slot entry children\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation\n    * `:directives` - directives associated with this slot entry","ref":"Surface.AST.SlotEntry.html#module-properties"},{"type":"type","title":"Surface.AST.SlotEntry.t/0","doc":"","ref":"Surface.AST.SlotEntry.html#t:t/0"},{"type":"module","title":"Surface.AST.SlotableComponent","doc":"An AST node representing a standard HTML tag","ref":"Surface.AST.SlotableComponent.html"},{"type":"module","title":"Properties - Surface.AST.SlotableComponent","doc":"* `:module` - the component module\n    * `:type` - the type of component (i.e. Surface.LiveComponent vs Surface.Component)\n    * `:slot` - the name of the slot that this component is for\n    * `:let` - the `:let` expression\n    * `:props` - the props for this component\n    * `:directives` - any directives to be applied to this tag\n    * `:children` - the tag children\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.SlotableComponent.html#module-properties"},{"type":"type","title":"Surface.AST.SlotableComponent.t/0","doc":"","ref":"Surface.AST.SlotableComponent.html#t:t/0"},{"type":"module","title":"Surface.AST.SubBlock","doc":"An AST node representing a generic sub-block.","ref":"Surface.AST.SubBlock.html"},{"type":"module","title":"Properties - Surface.AST.SubBlock","doc":"* `:name` - name of the block\n    * `:expression` - the expression passed to block\n    * `:children` - children AST nodes\n    * `:meta` - compile meta\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.SubBlock.html#module-properties"},{"type":"type","title":"Surface.AST.SubBlock.t/0","doc":"","ref":"Surface.AST.SubBlock.html#t:t/0"},{"type":"module","title":"Surface.AST.Tag","doc":"An AST node representing a standard HTML tag","ref":"Surface.AST.Tag.html"},{"type":"module","title":"Properties - Surface.AST.Tag","doc":"* `:element` - the element name\n    * `:attributes` - the attributes for this tag\n    * `:directives` - any directives to be applied to this tag\n    * `:children` - the tag children\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.Tag.html#module-properties"},{"type":"type","title":"Surface.AST.Tag.t/0","doc":"","ref":"Surface.AST.Tag.html#t:t/0"},{"type":"module","title":"Surface.AST.VoidTag","doc":"An AST node representing a void (empty) HTML tag","ref":"Surface.AST.VoidTag.html"},{"type":"module","title":"Properties - Surface.AST.VoidTag","doc":"* `:element` - the element name\n    * `:attributes` - the attributes for this tag\n    * `:directives` - any directives to be applied to this tag\n    * `:meta` - compilation meta data\n    * `:debug` - keyword list indicating when debug information should be printed during compilation","ref":"Surface.AST.VoidTag.html#module-properties"},{"type":"type","title":"Surface.AST.VoidTag.t/0","doc":"","ref":"Surface.AST.VoidTag.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.12.0 (2024-09-24) - Changelog","doc":"* Add column information to diagnostics to improve error/warning messages during compiler validation\n* Add compiler option `:generate_assets` to switch on/off component-related CSS/JS generation\n* Fix warning line when using  `@deprecated` on function components\n* Raise compile error when catalogue module cannot be found","ref":"changelog.html#v0-12-0-2024-09-24"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate components ` `, ` ` and ` `, in favor of `<.link>`\n* Deprecate component ` ` as support for scoped-aware context is planned to be removed in `v0.13`.","ref":"changelog.html#deprecations"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Move all form-related components, including ` ` and all inputs to a separate `surface_form_helpers` project.\n  For more info, see [Migration Guide](https://github.com/surface-ui/surface/blob/main/MIGRATING.md#migrating-to-v012x).\n* Rename `Surface.Catalogue.Example` to `Surface.Catalogue.LiveExample` as its use now should only be required in\n  live examples that update assigns through `handle_event` calls\n* Remove deprecated slot attributes `for`, `name` and `ìndex`\n* Remove deprecated slot `:args` option and directive","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.11.5 (2024-09-04) - Changelog","doc":"* Support Liveview `~> 0.20.15` (#749)","ref":"changelog.html#v0-11-5-2024-09-04"},{"type":"extras","title":"v0.11.4 (2024-04-09) - Changelog","doc":"* Add support for Liveview `~> 0.20.10` (#743)","ref":"changelog.html#v0-11-4-2024-04-09"},{"type":"extras","title":"v0.11.3 (yyyy-mm-dd) - Changelog","doc":"* Restrict phoenix_live_view version up to `0.20.9` due to compatibility\n  issues introduced by phoenix_live_view `0.20.10` regarding debug annotations.","ref":"changelog.html#v0-11-3-yyyy-mm-dd"},{"type":"extras","title":"Soft Deprecations - Changelog","doc":"* `Link` has been deprecated in favor of liveview's built-in `<.link>`.\n    See   for more info.","ref":"changelog.html#soft-deprecations"},{"type":"extras","title":"v0.11.2 (2024-02-19) - Changelog","doc":"* Add support for Liveview >= `v0.20` (#714)\n* Add support for debug annotations using liveview's `debug_heex_annotations` config.\n* Avoid using deprecated `hidden_inputs_for` in `HiddenInputs` (#716)\n* Update Tailwind patcher for phx_new >= 1.7.8 (#724)\n* Add a Dockerfile patcher for `surface.init` (#727)\n* Fix error related to `_persistent_id` (#716)\n* Fix warning about negative steps in Elixir 1.16 (#718 and #722)\n* Fix `Malformed HTML in rendered.js` error\n* Fix error when passing special chars as literals inside an expression","ref":"changelog.html#v0-11-2-2024-02-19"},{"type":"extras","title":"Soft Deprecations - Changelog","doc":"* `LivePatch` and `LiveRedirect` have been deprecated in favor of liveview's built-in `<.link>`.\n    See   for more info.\n* The `Form` component along with all form inputs will be moved to a separate project and\n    later deprecated in favor of their LiveView built-in counterparts.\n* Scope-aware context as described in   will\n    be deprecaded and fully removed in `v0.13`. The reason is the lack of built-in support for the\n    feature in Liveview itself, which leads to inefficient diff-tracking when using it. Global context\n    related functions like `Context.put` and `Context.get`, as well as the `:from_context` option, will\n    be kept as their use don't affect diff-tracking negatively.","ref":"changelog.html#soft-deprecations"},{"type":"extras","title":"v0.11.1 (2023-11-07) - Changelog","doc":"* Warn when module given to `catalogue_test` doesn't exist or is not a component (#711)\n* Fix `inputs_for/3` warnings (#703)","ref":"changelog.html#v0-11-1-2023-11-07"},{"type":"extras","title":"v0.11.0 (2023-06-02) - Changelog","doc":"* Add support for Liveview `v0.19` (#692)\n* Fix `Context.get/put` in LiveComponent's `update` callback (#691)\n* Fix issue when passing `css_class` props inside vanilla LV components' slots (#680)","ref":"changelog.html#v0-11-0-2023-06-02"},{"type":"extras","title":"v0.10.0 (2023-04-18) - Changelog","doc":"* Add support for Liveview `v0.18.8`\n* Add support for Phoenix `v1.7`\n* Introduce an optional `css_variant` option to `prop` and `data` to support auto-generation of CSS (tailwind)\n    variants based on their values\n* Add `embed_sface/1` macro to allow adding `.sface` template as a function component\n* Collect and generate `@import` entries from components to the top of the file to adhere to the CSS spec\n* Fix CSS scope so it can be shared by all function components in the same module, avoiding unnecessary use of `:deep`","ref":"changelog.html#v0-10-0-2023-04-18"},{"type":"extras","title":"v0.9.5 (2024-04-09) - Changelog","doc":"Improves compatibility with minor dependency versions used by Surface v0.9, this will improve the upgrade (`mix deps.update --all`) of apps that depend on this version.\n\n* Compatibility with `phoenix_live_view ~> 0.18.18`\n* Compatibility with `phoenix ~> 1.6.16`\n* Compatibility with `phoenix_html ~> 3.3.1`","ref":"changelog.html#v0-9-5-2024-04-09"},{"type":"extras","title":"v0.9.4 (2023-02-15) - Changelog","doc":"* Update `phoenix_live_view` to `v0.18.14`\n* Fix warnings on forms\n* Fix warning on unknown props of function components\n* Remove dependency on `Mix.env()` on re-compiling components at runtime","ref":"changelog.html#v0-9-4-2023-02-15"},{"type":"extras","title":"v0.9.3 (2023-02-09) - Changelog","doc":"* Fix error when trying to add the caller's scope_id attr to a MacroComponent","ref":"changelog.html#v0-9-3-2023-02-09"},{"type":"extras","title":"v0.9.2 (2023-02-06) - Changelog","doc":"* Compatibility with Liveview >= `v0.18.5`\n* Optimize static props for diff tracking\n* Fix child components' root DOM element not belonging to the parent's scope (#670)\n* Fix missing shortdoc on `surface.init` prevents task discovery from `mix help` (#666)","ref":"changelog.html#v0-9-2-2023-02-06"},{"type":"extras","title":"v0.9.1 (2022-09-26) - Changelog","doc":"* Fix dialyzer errors","ref":"changelog.html#v0-9-1-2022-09-26"},{"type":"extras","title":"v0.9.0 (2022-09-23) - Changelog","doc":"* Support Liveview `v0.18`","ref":"changelog.html#v0-9-0-2022-09-23"},{"type":"extras","title":"v0.8.4 (2022-09-26) - Changelog","doc":"* Fix dialyzer errors","ref":"changelog.html#v0-8-4-2022-09-26"},{"type":"extras","title":"v0.8.3 (2022-09-22) - Changelog","doc":"* Add `:global` pseudo-class to the Scoped CSS's API\n* Declare props `container`, `session` and `sticky` on `Surface.LiveView`\n* Fix CSS tokenizer not handling empty strings\n* Fix CSS parser for declarations with commas or variants\n* Fix scoped styles on void elements\n* Fix dialyzer issue in EExEngine (#655)","ref":"changelog.html#v0-8-3-2022-09-22"},{"type":"extras","title":"v0.8.2 (2022-09-16) - Changelog","doc":"* Remove compile-time deps from components, avoiding unnecessary recompilation\n    of extra files due to transitive deps (#653)\n* Fix components oversized beam files (#651)\n* Fix error on layout templates containing ` `","ref":"changelog.html#v0-8-2-2022-09-16"},{"type":"extras","title":"v0.8.1 (2022-09-02) - Changelog","doc":"* Fix surface compiler when setting a different `css_output_file` (#646)\n* Fix formatter for `:debug`","ref":"changelog.html#v0-8-1-2022-09-02"},{"type":"extras","title":"v0.8.0 (2022-09-01) - Changelog","doc":"* Support scoped CSS styles for components using either inline ` ` tags or colocated\n    `.css` files (#621)\n* Add `render_sface/1` to allow overriding `render/1` and compute/update assigns when using\n    external `.sface` files\n* Add `--tailwind` option to `mix surface.init` to bootstrap a project with TailwindCSS support\n* Add `--layouts` option to `mix surface.init` to replace `.heex` layout files with corresponding\n    `.sface` files\n* Add `--web-module` option to `mix surface.init` to override the default web module (#638)\n* Support using the `:hook` directive to point to the `default` hook exported in the `.hooks.js` file\n* Add option `from_context` to `prop` and `data` to allow setting values directly from the context\n* Add `Context.put/3` and `Context.get/2` to allow manipulating the context inside\n    lifecycle callbacks and `render/1`\n* Add prop `context_put` to `<#slot>` to pass context values directly to a slot without propagating\n    context values to other components\n* Add config `:propagate_context_to_slots` to restrict context propagatiion, optimizing diff tracking\n* Add functions `Context.copy_assign/3`, `Context.maybe_copy_assign/3` and `Context.maybe_copy_assign!/3`\n* Add `catalogue_test/1` macro to generate basic tests for catalogue examples and playgrounds\n* Add module `Surface.Catalogue.Examples` to allow defining multiple stateless examples\n    in a single module\n* Support editing slot values as text in playgrounds (Catalogue)\n* Fix context propagation in dynamic components\n* Fix context propagation in recursive components\n* New API for Slot arguments and generator (#613)","ref":"changelog.html#v0-8-0-2022-09-01"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate ` ` in favor of declarative option `from_context`\n* Slots (#613)\n  * Option `:args` has been deprecated, use\n    * `slot arg: :string` instead of `slot args: [:name]`\n    * `slot arg: %{name: :string, age: number}` instead of `slot args: [:name, :age]`\n  * Attribute `for` has been deprecated, use `<#slot {@header}>` instead of`<#slot for={@header}>`\n  * Attributes `name` and `index` have been deprecated, use `<#slot {col}>` instead of`<#slot name={\"col\"} index={index}>`\n  * Directive `:args` has been deprecated, use\n    * `<#slot {@default, name}>` instead of `<#slot :args={name: name}>`\n    * `<#slot {@default, name: name, age: age}>` instead of `<#slot :args={name: name, age: age}>`","ref":"changelog.html#deprecations"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Drop support for Elixir < `v1.12`\n* Context values are no longer automatically propagated through slots. Components that need to\n    pass values to the parent scope via slots must explicitly set `propagate_context_to_slots: true`\n    in their configuration\n* Slots (#613)\n  * New option `:generator_prop` use `slot default, generator_prop: :items` instead of `slot default, args: [item: ^items]`, associated prop `:items` must be of type `:generator`\n  * New attribute `generator_value` use `<#slot generator_value={item} />` instead of `<#slot :args={item: item} />`\n  * `<#template slot=\"slot_name\">` has been removed in favor of `<:slot_name>` (#575)","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.7.6 (2022-09-05) - Changelog","doc":"* Support Elixir `v1.14`","ref":"changelog.html#v0-7-6-2022-09-05"},{"type":"extras","title":"v0.7.5 (2022-07-21) - Changelog","doc":"* Support using vanilla phoenix function components with slots in surface templates","ref":"changelog.html#v0-7-5-2022-07-21"},{"type":"extras","title":"v0.7.4 (2022-04-18) - Changelog","doc":"* Optimize the surface compiler for assets generation\n* Improve support for JS hooks in umbrella projects (#591)\n* Suppress `Mix.Tasks.Format` behaviour warning on Elixir < `v0.13`","ref":"changelog.html#v0-7-4-2022-04-18"},{"type":"extras","title":"v0.7.3 (2022-03-18) - Changelog","doc":"* Fix loading component prop's default values","ref":"changelog.html#v0-7-3-2022-03-18"},{"type":"extras","title":"v0.7.2 (2022-03-17) - Changelog","doc":"* Support more extensions other than `.js` as colocated hooks (`jsx`, `ts` and `tsx`) (#576)\n* Update the `surface.init` task to set up the catalogue to `v0.4`","ref":"changelog.html#v0-7-2-2022-03-17"},{"type":"extras","title":"v0.7.1 (2022-02-17) - Changelog","doc":"* Fix wrong target handling in forms\n* Fix setting `@moduledoc false` in catalogue examples (#565)\n* Support Inputs' property `for` as string (#564)","ref":"changelog.html#v0-7-1-2022-02-17"},{"type":"extras","title":"v0.7.0 (2022-01-13) - Changelog","doc":"* Support Liveview `v0.17`\n* Support rendering `.sface` templates for regular (dead) views (#543)\n* Support passing properties to slots using the shorthand format, e.g. `<:col label=\"Name\">`\n* Add built-in formatter supporting integration with `mix format` (#535)\n* New ` ` component to inject dynamic live components (#518)\n* Optimize rendering of HTML class attributes literals so they can be treated as static text\n* Add property `for` to `<#slot/>` so it can render the slot content directly (without using `index`)","ref":"changelog.html#v0-7-0-2022-01-13"},{"type":"extras","title":"v0.6.1 (2021-10-26) - Changelog","doc":"* Add `surface_formatter` dependency to `mix.exs` when running `mix surface.init` (#507)\n* Allow `Inputs` component inside the `Field` component (#492)\n* Fix using context with external `.sface` templates (#511)\n* Fix attribute name conversion (#512)","ref":"changelog.html#v0-6-1-2021-10-26"},{"type":"extras","title":"v0.6.0 (2021-10-21) - Changelog","doc":"* Compatibility with Phoenix `v1.6` and Liveview `v0.16`\n* New `mix surface.init` task\n* Add support for function components\n* Add support for dynamic function components via ` `\n* Add support for recursive function components\n* Optimize change tracking for contexts\n* Fix race condition when compiling tests\n* Fix recompilation of used components","ref":"changelog.html#v0-6-0-2021-10-21"},{"type":"extras","title":"v0.5.1 (2021-07-13) - Changelog","doc":"* Add property `values` to form inputs\n* Handle doctype as text\n* Improve error message when `default_translator` is not configured for `ErrorTag` (#449)\n* Raise on invalid attribute/directive in `<#slot>` (#456)\n* Raise error on `{#case}` without `{#match}` (#443)\n* Raise on blocks without expression\n* Fix error line on missing closing tag","ref":"changelog.html#v0-5-1-2021-07-13"},{"type":"extras","title":"v0.5.0 (2021-06-17) - Changelog","doc":"* Add `<:slotname>` shorthand for `<#template slot=\"slotname\">`\n* Introduce block expressions for surface templates (e.g., `{#if}..{/if}`)\n* Introduce `{#if}` block expression with support for `{#elseif}` and `{#else}` sub blocks\n* Introduce `{#for}` block expression with support for `{#else}` sub block\n* Introduce `{#unless}` block expression\n* Introduce new shorthand notation for dynamic attributes/props using the `{... }` tagged expression\n* Introduce new shorthand notation for attribute assignment using the `{= }` tagged expression\n* Support private comments using `{!--  --}` for comments that are not supposed to hit the browser\n* Introduce `s-` prefix as an alternative to `:` for directives (i.e. `s-if` and `:if` are now equivalent)\n* Introduce `:values` directive for generating multiple `phx-value-` attributes\n* Added a convert task to aid migrating to the new syntax\n* Evaluate literal attribute values at compile time instead of runtime\n* Fix compile error when using single quotes as attribute value delimiters\n* Add `quote_surface/2` macro to generate Surface AST from template snippets.","ref":"changelog.html#v0-5-0-2021-06-17"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"* Replace the sigil `~H` with `~F` to avoid conflict with `HEEx`\n* Replace interpolation delimiters `{{` and `}}` with `{` and `}`\n* Remove support for interpolation inside ` ... ` and ` ... ` nodes\n* ErrorTag: Renamed prop `phx_feedback_for` to `feedback_for`\n* Slot directive `:props` has been renamed to `:args`\n* Option `:props` for the `slot/2` macro has been renamed to `:args`\n* The use of ` ` has been removed in favor of `<#template>`\n* The use of ` ` has been removed in favor of `<#slot>`\n* The use of ` ` has been removed in favor of `{#if}...{/if}`\n* The use of ` ` has been removed in favor of `{#for}...{/for}`\n* `MacroComponent.eval_static_props!/3` evaluates and returns only props with option `static: true`","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Support for passing non-string attribute values as literals (i.e. `selected=true` or `tabindex=3`) has been removed.\n    Any non-string value should be passed as an expression (i.e. `selected={true}`)","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.4.1 (2021-05-26) - Changelog","doc":"* Fix warning on Phoenix Live View >= 1.15.6","ref":"changelog.html#v0-4-1-2021-05-26"},{"type":"extras","title":"v0.4.0 (2021-05-01) - Changelog","doc":"* Call render when defined in slotable components (#283)\n* Support defining form fields as strings. Consequently, fields defined as literal strings will\n    no longer be auto-converted to `:atom` and will keep the original value (#319)\n* Deprecate auto-conversion of attribute values passed as string literals into atoms\n* Do not encode HTML entities when passing attribute values as string literals (#323)\n* Extract the Markdown macro component in its repository (#316)\n* Renamed `Surface.Components.Button` to `Surface.Components.Link.Button` (#350)","ref":"changelog.html#v0-4-0-2021-05-01"},{"type":"extras","title":"v0.3.2 (2021-03-19) - Changelog","doc":"* Warn if prop is required and has default value (#282)\n* Warn if slot is required and has a fallback content (#296)\n* Warn on `LiveComponent` with another `LiveComponent` as root\n* Support escaped three double-quotes in `Markdown` content\n* Improve `Label` component compatibility with Phoenix `label/2` (#284)\n* Update props according to new types (#297)\n* Fix copying JS hooks multiple times (#294)","ref":"changelog.html#v0-3-2-2021-03-19"},{"type":"extras","title":"v0.3.1 (2021-03-05) - Changelog","doc":"* Fix `index.js` generation when no hooks are available\n* Fix loading hooks from dependencies\n* Support ` ` with scheme (#273)","ref":"changelog.html#v0-3-1-2021-03-05"},{"type":"extras","title":"v0.3.0 (2021-02-24) - Changelog","doc":"* Autoload JS hooks via new surface compiler (#271)\n* New ` ` and ` ` implementation without depending on `content_tag` to allow receiving\n    child components in slots (#264)\n* Don't validate undefined assigns outside render (#263)\n* Load subject's default props values before sending them to playgrounds","ref":"changelog.html#v0-3-0-2021-02-24"},{"type":"extras","title":"v0.2.1 (2021-02-01) - Changelog","doc":"* Allow different catalogue options for examples and playgrounds","ref":"changelog.html#v0-2-1-2021-02-01"},{"type":"extras","title":"v0.2.0 (2021-01-27) - Changelog","doc":"* Introduce new testing API using `render_surface/1`\n* Add experimental support to create examples and playgrounds for catalogues\n* Raises compile error if slots are not declared\n* Raises compile error on duplicate built-in assign\n* Allow defining the assign name for slot through the :as option (#230)\n* Implement the `:show` directive via hidden attribute (#244)\n* Add new ` ` component\n* Remove default value from `Form` method prop\n* Reintroduce opts prop for the ` ` component\n* Fix markdown syntax warning in ` ` docs\n* Fix error when using :if + :props in slots (#224)\n* Fix line offset when using single-line `~H` variants (#246)\n* Fix UnicodeConversionError when using string literals inside interpolation","ref":"changelog.html#v0-2-0-2021-01-27"},{"type":"extras","title":"v0.1.1 (2020-11-28) - Changelog","doc":"* Add explicit props for the main opts of Checkbox, Select, MultipleSelect, FileInput and Form (#215).\n* Add new `slot_assigned?/1` macro to check if a slot has been filled in (#211).\n* Fix attribute value encoding","ref":"changelog.html#v0-1-1-2020-11-28"},{"type":"extras","title":"v0.1.0 (2020-11-23) - Changelog","doc":"* Update liveview to v0.15\n* Add new `Surface.Components.Form.ErrorTag` to render error messages in forms (#199).\n* Disable validation for required props if `:props` is passed (#204)","ref":"changelog.html#v0-1-0-2020-11-23"},{"type":"extras","title":"v0.1.0-rc.2 (2020-11-04) - Changelog","doc":"* Update liveview to latest v0.15-dev (f986171)\n* New wrapper components `Surface.Components.{For, If}` for when the `:for` and `:if` directives aren't sufficient (#184)\n* Allow double braces within interpolation (#171)\n* Add new `Surface.Components.FieldContext` to support form fields without wrapping divs (#172)\n* Improve error message for unloaded modules (#174)\n* Fix issue with `:for` modifiers on components (#176)\n* Expose form instance as slot prop on `Surface.Components.Form` (#183)\n* Don't initialize data assigns without default value (#195)","ref":"changelog.html#v0-1-0-rc-2-2020-11-04"},{"type":"extras","title":"v0.1.0-rc.1 (2020-10-21) - Changelog","doc":"* Fix support for Elixir >= v1.11\n* Update liveview to latest v0.15-dev (597c5dd)\n* Add undefined assign check for `Surface.{LiveComponent,Component,LiveView}`\n* New form controls wrappers: ` ` and ` `.\n* Force recompilation of the parent component after fixing errors on any of its children.","ref":"changelog.html#v0-1-0-rc-1-2020-10-21"},{"type":"extras","title":"v0.1.0-rc.0 (2020-10-06) - Changelog","doc":"* Update LiveView to v0.15-dev (0f592a4).\n* Make ` ` mandatory instead of `inner_content`.\n* Add attribute `index` to ` ` to allow rendering individual named slot items.\n* Rename macro `property` to `prop`.\n* Remove macro `context` and add a ` ` component to be used instead.\n* Rename directives `:on-phx-[event]` to `:on-[event]`.\n* Add support for co-located template files using `.sface` suffix.\n* Add `Surface.init/1` to initialize internal assigns when not using `Surface.LiveView`.\n* Add `:props` directive to pass dynamic props to a component.\n* Add `:attrs` directive to pass dynamic attributes to a tag.\n* Add new modifiers `index` and `with_index` for `:for`.\n* Update html tag generation to remove the tag if it's value computes to `nil`.\n* Add support for a `transform/1` callback to allow components to manipulate its\n    own node at compile-time.\n* New form controls: ` `, ` `, ` `, ` `,\n    ` `, ` ` and ` `.","ref":"changelog.html#v0-1-0-rc-0-2020-10-06"},{"type":"extras","title":"v0.1.0-alpha.2 (2020-06-09) - Changelog","doc":"* New Markdown component\n* New Link component\n* New form components Form, Field, TextArea, Label, TextInput, RadioButton,\n    HiddenInput, ColorInput, DateInput, DateTimeLocalInput, EmailInput, NumberInput,\n    PasswordInput, RangeInput, SearchInput, TelephoneInput, TimeInput, UrlInput,\n    Reset and Submit.\n* Automatically define a required :id property for live components that implement\n    `handle_event/3`\n* New config API for components\n* Update LiveView to v0.13","ref":"changelog.html#v0-1-0-alpha-2-2020-06-09"},{"type":"extras","title":"v0.1.0-alpha.1 (2020-04-13) - Changelog","doc":"* Add support for slots\n* Add built-in LivePath and LiveRedirect components\n* Drop automatic camel-to-kebab conversion for CSS class name\n* Drop support for `inner_content.()`. Use `inner_content.([])` instead\n* Update LiveView to v0.11.1","ref":"changelog.html#v0-1-0-alpha-1-2020-04-13"},{"type":"extras","title":"v0.1.0-alpha.0 (2020-02-26) - Changelog","doc":"* Initial alpha release","ref":"changelog.html#v0-1-0-alpha-0-2020-02-26"},{"type":"extras","title":"Migrating to v0.12.x","doc":"# Migrating to `v0.12.x`","ref":"migrating.html"},{"type":"extras","title":"Forms - Migrating to v0.12.x","doc":"Component ` ` and all its related input components (` `, ` `, etc.) have been\ndeprecated and moved to a separate project called `surface_form_helpers`. The reason those components were created\nin the first place was to add support for [scope-aware contexts](https://www.surface-ui.org/contexts#scope-aware-context)\nto forms. Since this feature has been deprecated due to the lack of built-in support in Liveview, we strongly\nrecommend using the new built-in [<.form>](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#form/)\ncomponent, along with [Phoenix.HTML.Form](https://hexdocs.pm/phoenix_html/Phoenix.HTML.Form.html) and\n[Phoenix.HTML.FormField](https://hexdocs.pm/phoenix_html/Phoenix.HTML.FormField.html) structs. This new Liveview API\nis way more efficient regarding diff-tracking and should be the default way to desing forms.\n\nIf you're not able to update all your forms at once, `surface_form_helpers` can be used as a safe fallback so you can\ngradually replace them.","ref":"migrating.html#forms"},{"type":"extras","title":"Using `surface_form_helpers` - Migrating to v0.12.x","doc":"```elixir\ndef deps do\n  [\n    {:surface_form_helpers, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"migrating.html#using-surface_form_helpers"},{"type":"extras","title":"Surface Catalogue - Migrating to v0.12.x","doc":"In case you're using `surface_catalogue` and have examples using `Surface.Catalogue.Example`, you need to\nrename those with `Surface.Catalogue.LiveExample`. Pay attention that you should use `LiveExample` mostly\nif your example requires manipulating state (data) through `handle_event` callbacks. For stateless examples,\nuse `Surface.Catalogue.Examples` instead, as it allows defining multiple examples on a single module.\n\n# Migrating from `v0.7.x` to `v0.8.x`\n\nHistorically, most of the updates that require changes in your code, can be automatically\ndone by `mix surface.convert`, however, as we move forward to unify the API with Liveview, some of the\nchanges in `v0.8` may require user intervention as the converter might not be able to automatically\npatch the project. For such cases, we adjusted the compiler to provide assertive warnings to help you\nwith the migration process. The main changes of that kind are related to the context and slot APIs.","ref":"migrating.html#surface-catalogue"},{"type":"extras","title":"The new Context API - Migrating to v0.12.x","doc":"The context API have been extended and fully redesigned to improve its use and make it more friendly for\ndiff tracking. The compiler is able now to detect many cases where the use of contexts might impact\nperformance and suggest one or more alternative approaches to achieve the same goal. We recommend you\nto carefully read each warning with care and follow the instructions that best suits your particular case.\n\nAside from the warnings, the only breaking change is that context values are no longer automatically\npropagated through slots. Components that need to pass values to the parent scope via slots must\nexplicitly set `propagate_context_to_slots: true` in `config.exs`:\n\n```elixir\nconfig :surface, :components, [\n  {Surface.Components.Form, propagate_context_to_slots: true},\n]\n```\n\nThe compiler will raise an error whenever it finds a component that can potentially propagate context\nvalues through slots. If you don't want to use contexts at all, you need to set `propagate_context_to_slots`\nto `false` to suppress the error for that component.\n\n> **NOTE:** The following built-in Surface components are already configured to propagate context to slots:\n> `Surface.Components.Form,`, `Surface.Components.Form.Field`, `Surface.Components.Form.FieldContext` and\n> `Surface.Components.Form.Inputs`.","ref":"migrating.html#the-new-context-api"},{"type":"extras","title":"The new Slot API - Migrating to v0.12.x","doc":"The slot API for arguments and generators have changed to make it similar to Liveview.","ref":"migrating.html#the-new-slot-api"},{"type":"extras","title":"Rendering Slots - Migrating to v0.12.x","doc":"Previously the reference was given in a combination of the `for`, `name` and `index` arguments.\nNow the slot reference is given as the first argument of the root property: `<#slot {@name} />`.\n\n> The converter doesn't update any `<#slot>` that has the `index` property.","ref":"migrating.html#rendering-slots"},{"type":"extras","title":"Slot argument(s) - Migrating to v0.12.x","doc":"Previous versions of Surface only allowed the syntax `slot default, args: [:index]` and `<#slot arg={index: 10}>`,\nin other words we always required the argument to be a map. Now the argument can be any value and in `:let` we can\npattern match more freely and be compatible with Liveview slots/components.\n\nNow the slot argument is given as the second argument of the root property: `<#slot {@name, argument} />`.","ref":"migrating.html#slot-argument-s"},{"type":"extras","title":"Slot generator - Migrating to v0.12.x","doc":"The changes to generator are too complex for the converter to handle, the compiler errors/warnings will guide you through\nthe needed changes.\n\n```elixir\n# before\nprop items, :list\nslot cols, args: [item: ^items]\n~F\"\"\"\n{#for item <- @items}\n  <#slot :args={item: item} />\n{/for}\n\"\"\"\n```\n\n```elixir\n# after\nprop items, :generator\nslot cols, generator_prop: :items\n~F\"\"\"\n{#for item <- @items}\n  <#slot generator_value={item} />\n{/for}\n\"\"\"\n```","ref":"migrating.html#slot-generator"},{"type":"extras","title":"Running `mix surface.convert` - Migrating to v0.12.x","doc":"This guide provides detailed instructions on how to run the built-in converter to automatically\napply required changes to your project's source code to make it compatible\nwith Surface `v0.8`.\n\n> **NOTE:**  The current converter was designed to run against projects depending on Surface `>= v0.7.x`.\n> If you're using an older version, you should first update it to each one of the previous versions\n> all the way to `v0.7`. See: [previous versions](#previous-versions)","ref":"migrating.html#running-mix-surface-convert"},{"type":"extras","title":"Limitations of the converter - Migrating to v0.12.x","doc":"* By design, the converter doesn't touch Surface code inside documentation or macro components. If you have\n  any code written inside `<!-- -->` or `<#Raw>...</#Raw>`, you need to convert it manually.\n\n* Running the converter on a project that has already been converted may generate invalid code. If anything goes\n  wrong with the conversion, make sure you revert the changes before running it again.","ref":"migrating.html#limitations-of-the-converter"},{"type":"extras","title":"Before converting the project - Migrating to v0.12.x","doc":"1. Make sure you have committed your work or have a proper backup before running the converter. It may touch\n  a lot of files so it's recommended to have a safe way to rollback the changes in case anything goes wrong.\n\n  2. Check your dependencies. In case your project depends on a library using an older Surface version, it might start\n  emitting warnings or even fail to compile after updating Surface. If that's the case, please consider running\n  the converter against it and submitting a PR with the updated code. The steps to convert a dependency are the\n  same described in this guide.\n\n  3. Check that your `.formatter` has the configuration about the components you want to convert.\n  The [plugin documentation](https://hexdocs.pm/surface/Surface.Formatter.Plugin.html#module-formatter-exs-setup) has the instructions.","ref":"migrating.html#before-converting-the-project"},{"type":"extras","title":"Steps to run the converter - Migrating to v0.12.x","doc":"Update `mix.exs` to use the new version:\n\n```elixir\n  defp deps do\n    [\n      {:surface, \"~> 0.8.0\"},\n      ...\n    ]\n  end\n```\n\nCompile the dependencies:\n\n```shell\nmix clean && mix deps.get && mix deps.compile\n```\n\nCompile the project and fix all compilation errors, changes to the slot generator API are not handled by the converter.\n\n```shell\nmix compile\n```\n\nRun the converter:\n\n```shell\nmix surface.convert\n```\n\nCompile the converted project:\n\n```shell\nmix compile\n```","ref":"migrating.html#steps-to-run-the-converter"},{"type":"extras","title":"Expected changes - Migrating to v0.12.x","doc":"| Subject                       | Examples (Old syntax -> New syntax)                                                           |\n| ----------------------------- | --------------------------------------------------------------------------------------------- |\n| Slot Entry                    | `<#template>` -> `<:default>`   |\n| Slot Entry                    | `<#template slot=\"header\">` -> `<:header>`   |\n| `<#slot for>`                  | `<#slot for={@name}>` -> `<#slot {@name}>`                                             |\n| `<#slot name>`                 | `<#slot name=\"name\">` -> `<#slot {@name}>`                                             |\n| `<#slot :args>`                | `<#slot :args={name: \"Joe\", age: \"32\"}>` -> `<#slot {@default, name: \"Joe\", age: 35}>` |","ref":"migrating.html#expected-changes"},{"type":"extras","title":"Reporting issues - Migrating to v0.12.x","doc":"In case you run into any trouble while running the converter, please open an issue at  \nproviding detailed information about the problem, including the error message (if any) and a snippet of the\nrelated code.","ref":"migrating.html#reporting-issues"},{"type":"extras","title":"Previous versions - Migrating to v0.12.x","doc":"* Migrating from `v0.6.x` to `v0.7.x`\n\n  No converter was needed.\n\n* [Migrating from `v0.5.x` to `v0.6.x`](https://github.com/surface-ui/surface/blob/v0.7/MIGRATING.md#migrating-from-v05x-to-v06x)\n\n  Surface `v0.6.x` relies on the Liveview features available since `v0.16`. The main change\n  from the user perspective is that the stateless `Surface.Component` now is built on top of\n  `Phoenix.Component` instead of `Phoenix.LiveComponent`. This means the `mount/1`, `preload/1`\n  and `update/2` callbacks are no longer available. If you initialize any assign or compute\n  any value using those callbacks, you need to replace them with one of the new\n  [assign helpers](https://hexdocs.pm/phoenix_live_view/Phoenix.Component.html#module-assigns).\n\n* [Migrating from `v0.4.x` to `v0.5.x`](https://github.com/surface-ui/surface/blob/v0.5/MIGRATING.md#migrating-from-v04x-to-v05x)\n\n  Surface `v0.5.0` introduces a new syntax which requires migrating components written in previous versions.\n  In order to make the migration process as smooth as possible, Surface `v0.5.x` ships with a converter that\n  can automatically translate the old syntax into the new one.","ref":"migrating.html#previous-versions"},{"type":"extras","title":"MIT License","doc":"# MIT License\n\nCopyright (c) 2019 Marlus Saraiva\n\nPermission is hereby granted, free of charge, to any person obtaining\na copy of this software and associated documentation files (the\n\"Software\"), to deal in the Software without restriction, including\nwithout limitation the rights to use, copy, modify, merge, publish,\ndistribute, sublicense, and/or sell copies of the Software, and to\npermit persons to whom the Software is furnished to do so, subject to\nthe following conditions:\n\nThe above copyright notice and this permission notice shall be\nincluded in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,\nEXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF\nMERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND\nNONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE\nLIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION\nWITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}